import sys,ast, copy

# sys.path.append("/mnt/zejun/smp/code1/")

'''
变量映射函数，
对于当前的赋值语句的左边变量在后面赋值语句的右边变量中存在依赖关系，对当前赋值语句建立Map, 之后对后面所有赋值语句的变量进行变量替换
记录 第i个赋值语句，遍历后面赋值表达式进行节点替换，然后简化为多赋值语句
'''


class RewriteName(ast.NodeTransformer):
    def __init__(self, Map_var):
        super(RewriteName, self).__init__()
        self.Map_var = Map_var
    def visit_Name(self, node):

        # print("visit_Subscript node: ", ast.unparse(node))
        if ast.unparse(node) in self.Map_var:
            # print("yes: ",ast.unparse(node))
            return self.Map_var[ast.unparse(node)]
        else:

            return node

    def visit_Attribute(self, node):
        if ast.unparse(node) in self.Map_var:
            # print("yes: ",ast.unparse(node))
            return self.Map_var[ast.unparse(node)]
        else:

            if isinstance(node.value, (ast.Attribute, ast.Name, ast.Subscript, ast.Slice)):
                node.value = self.visit(node.value)
            # if node.attr in self.Map_var:
            #     node.attr = self.Map_var[node.attr]
            return node

    def visit_Subscript(self, node):
        if ast.unparse(node) in self.Map_var:
            # print("yes: ",ast.unparse(node))
            return self.Map_var[ast.unparse(node)]
        else:

            if isinstance(node.value, (ast.Attribute, ast.Name, ast.Subscript, ast.Slice)):
                node.value = self.visit(node.value)
            if isinstance(node.slice, (ast.Attribute, ast.Name, ast.Subscript, ast.Slice)):
                node.slice = self.visit(node.slice)

            return node

    def visit_Slice(self, node):
        if ast.unparse(node) in self.Map_var:
            return self.Map_var[ast.unparse(node)]
        else:
            if isinstance(node.lower, (ast.Attribute, ast.Name, ast.Subscript, ast.Slice)):
                node.lower = self.visit(node.lower)
            if isinstance(node.upper, (ast.Attribute, ast.Name, ast.Subscript, ast.Slice)):
                node.upper = self.visit(node.upper)
            if isinstance(node.step, (ast.Attribute, ast.Name, ast.Subscript, ast.Slice)):
                node.step = self.visit(node.step)
            return node
def is_depend(pre_tar, next_tar):
        pre_tar_str = ast.unparse(pre_tar)
        for node in ast.walk(next_tar):
            if pre_tar_str == ast.unparse(node):
                return 1

        return 0
def transform_multiple_assign(old_ass_list):
    # print("ass_list: ",ass_list)
    ass_list=copy.deepcopy(old_ass_list)
    remove_ind_list=set([])
    Map_var=dict()
    for ind, pre_ass in enumerate(ass_list[:-1]):

        for ind_e, next_ass in enumerate(ass_list[ind+1:]):
            # print("pre_ass,next_ass: ", ast.unparse(pre_ass),ast.unparse(next_ass))
            pre_tar=pre_ass.targets[0]
            next_value=next_ass.value
            if is_depend(pre_tar, next_value):
                # print("is depend: ")
                remove_ind_list.add(ind)
                pre_value =pre_ass.value
                Map_var={**Map_var,ast.unparse(pre_tar):pre_value}
                # ass_list[ind_e+ind+1] =RewriteName(Map_var).visit(next_ass)

    left=[]
    right=[]
    # print("remove_ind_list: ",remove_ind_list,Map_var)
    for ind,assign in enumerate(ass_list):
        if ind in remove_ind_list:
            continue
        # for key in Map_var:
        #     print("value: ",ast.unparse(Map_var[key]))
        # print("old_assign: ", assign,ast.unparse(assign),Map_var)
        # assign.value=RewriteName(Map_var).visit(assign.value)
        assign.value=RewriteName(Map_var).visit(assign.value)

        # print("assign: ",ast.unparse(assign.value))
        assign_str=ast.unparse(assign)
        # print("assign_str: ",assign_str,assign_str.split("="))
        vars=ast.unparse(assign).split("=")
        left.append(vars[0])
        right.append(ast.unparse(assign.value))
    return ", ".join(left)+" = "+", ".join(right)



if __name__ == '__main__':
    code='''
# self._tmp_path = tmp_path
# self._config = config
# self.location = str(tmp_path)

# print("split1")
# user_check = self.run_function('file.get_user', [check])
# mode_check = self.run_function('file.get_mode', [check])
# print("split2")
# name = obj.find('name').text.strip()
# bbox = obj.find('bndbox')
# pts = ['xmin', 'ymin', 'xmax', 'ymax']
# bndbox = []
# print("split3")
tmp_real = data[ii]
tmp_imag = data[ii + 1]
data[ii] = data[jj]
data[ii + 1] = data[jj + 1]
data[jj] = tmp_real
data[jj + 1] = tmp_imag
'''
    # 对找到的代码片段不断进行转换直至不可以转换
    code_list=[
        ["user_check = self.run_function('file.get_user', [check])","mode_check = self.run_function('file.get_mode', [check])"],
        ["name = obj.find('name').text.strip()","bbox = obj.find('bndbox')"],
        ["self._tmp_path = tmp_path","self._config = config","self.location = str(tmp_path)"]]
    # while 1:
    ass_list=[]
    tree=ast.parse(code)
    for node in ast.walk(tree):
        if isinstance(node,ast.Assign):
            ass_list.append(node)
    new_code = transform_multiple_assign(ass_list)
    print(new_code)

    # for code_frag in code_list:
    #         new_code=transform_multiple_assign(code_frag)
    #         print(new_code)


















import sys,ast,os


abs_path=os.path.abspath(os.path.dirname(__file__))
abs_path=abs_path[:abs_path.index("extract_transform_complicate_code_new")]

sys.path.append(abs_path)
sys.path.append(abs_path+"extract_transform_complicate_code_new/")
sys.path.append(abs_path+"extract_transform_complicate_code_new/comprehension/")
import extract_compli_for_comprehension_logic
from RefactoringIdioms.extract_simp_cmpl_data import ast_util
from RefactoringIdioms.transform_c_s import transform_comprehension_set_compli_to_simple
import traceback
# from pathos.multiprocessing import ProcessingPool as newPool
'''
get comprehension node
Block=> Assign;...; For
For=>”for” ( For| If|If-else|body)
If=>”if” (body| For| If| If-else)
If-else=>”if” body “else” (body| If-else)
body=> (Assign|Call)*;Append*
并且里面只有一个写变量
for

    b.append()
    
    a.append(b)

'''
const_func_name="add"
const_empty=["set()"]

def get_set_compreh(content):
    code_pair_list=[]
    # print(content)

    try:

        file_tree = ast.parse(content)
        ana_py = ast_util.Fun_Analyzer()
        ana_py.visit(file_tree)
        # print("come here",ana_py.func_def_list)
        for tree, class_name in ana_py.func_def_list:
            # print("come method",ast.unparse(tree))
            try:
                me_name = tree.name
            except:
                me_name = ""
            new_code_list = extract_compli_for_comprehension_logic.get_complicated_for_comprehen_code_list(tree,
                                                                                                           content,
                                                                                                           ["set()"],
                                                                                                           "add")
            # print("come method", new_code_list,ast.unparse(tree))
            for ind, (for_node, assign_node, remove_ass_flag) in enumerate(new_code_list):
                new_code= transform_comprehension_set_compli_to_simple.transform(for_node, assign_node)
                complete_new_code=ast.unparse(new_code) if remove_ass_flag else ast.unparse(assign_node) + "\n"+ast.unparse(new_code)
                assign_node_content = ast.get_source_segment(content, assign_node)
                for_node_content = ast.get_source_segment(content, for_node)
                if remove_ass_flag:
                    line_list = []
                    line_list.append([(assign_node.lineno,assign_node.col_offset), (assign_node.end_lineno,assign_node.end_col_offset)])
                    line_list.append([(for_node.lineno,for_node.col_offset), (for_node.end_lineno,for_node.end_col_offset)])
                    code_pair_list.append([class_name, me_name, [assign_node_content, for_node_content],
                                           ['', ast.unparse(new_code)],
                                           line_list])
                else:
                    line_list = []
                    line_list.append([(for_node.lineno,for_node.col_offset), (for_node.end_lineno,for_node.end_col_offset)])
                    code_pair_list.append(
                        [class_name, me_name, [for_node_content], [ast.unparse(new_code)], line_list])

                # line_list = []
                # line_list.append([assign_node.lineno, assign_node.end_lineno])
                # line_list.append([for_node.lineno, for_node.end_lineno])
                # code_pair_list.append([class_name,me_name,ast.unparse(assign_node) + "\n" + ast.unparse(for_node), complete_new_code,line_list])
                #
        return code_pair_list

    except:
        traceback.print_exc()
        return code_pair_list

def get_part_set_compreh(content):
    code_pair_list = []
    # print(content)

    try:

        tree = ast.parse(content)
        ana_py = ast_util.Fun_Analyzer()
        if 1:
            # print("come method",ast.unparse(tree))
            try:
                me_name = tree.name
            except:
                me_name = ""
            new_code_list = extract_compli_for_comprehension_logic.get_complicated_for_comprehen_code_list(tree,
                                                                                                           content,
                                                                                                           [
                                                                                                               "set()"],
                                                                                                           "add")
            # print("come method", new_code_list,ast.unparse(tree))
            for ind, (for_node, assign_node, remove_ass_flag) in enumerate(new_code_list):
                new_code = transform_comprehension_set_compli_to_simple.transform(for_node, assign_node)
                complete_new_code = ast.unparse(new_code) if remove_ass_flag else ast.unparse(
                    assign_node) + "\n" + ast.unparse(new_code)
                assign_node_content = ast.get_source_segment(content, assign_node)
                for_node_content = ast.get_source_segment(content, for_node)
                if remove_ass_flag:
                    line_list = []
                    line_list.append([(assign_node.lineno, assign_node.col_offset),
                                      (assign_node.end_lineno, assign_node.end_col_offset)])
                    line_list.append([(for_node.lineno, for_node.col_offset),
                                      (for_node.end_lineno, for_node.end_col_offset)])
                    code_pair_list.append(['', me_name, [assign_node_content, for_node_content],
                                           ['', ast.unparse(new_code)],
                                           line_list])
                else:
                    line_list = []
                    line_list.append([(for_node.lineno, for_node.col_offset),
                                      (for_node.end_lineno, for_node.end_col_offset)])
                    code_pair_list.append(
                        ['', me_name, [for_node_content], [ast.unparse(new_code)], line_list])

                # line_list = []
                # line_list.append([assign_node.lineno, assign_node.end_lineno])
                # line_list.append([for_node.lineno, for_node.end_lineno])
                # code_pair_list.append([class_name,me_name,ast.unparse(assign_node) + "\n" + ast.unparse(for_node), complete_new_code,line_list])
                #
        return code_pair_list

    except:
        traceback.print_exc()
        return code_pair_list
        # break
# def save_one_repo(repo_name):
#     count=0
#     one_repo_chained_comparison_code_list = []
#     dict_file = dict()
#     for file_info in dict_repo_file_python[repo_name]:
#         file_path = file_info["file_path"]
#         file_html = file_info["file_html"]
#         # if file_html!="https://github.com/salesforce/policy_sentry/tree/master/policy_sentry/querying/all.py":#"https://github.com/bndr/pipreqs/tree/master/pipreqs/pipreqs.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/networkx/networkx/tree/master/networkx/readwrite/json_graph/adjacency.py":#"https://github.com/microsoft/nni/tree/master/nni/tools/nnictl/nnictl_utils.py":#"https://github.com/microsoft/nni/tree/master/nni/tools/nnictl/nnictl_utils.py":#:#"https://github.com/aws/aws-cli/tree/master/awscli/customizations/s3/subcommands.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":##"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/localstack/localstack/tree/master/localstack/utils/common.py":#"https://github.com/pytransitions/transitions/tree/master/transitions/extensions/nesting.py":##"https://github.com/HypothesisWorks/hypothesis/tree/master/hypothesis-python/examples/test_rle.py":
#         #     continue
#         # if file_html!="https://github.com/smicallef/spiderfoot/tree/master/spiderfoot/helpers.py":#"https://github.com/HypothesisWorks/hypothesis/tree/master/hypothesis-python/examples/test_rle.py":
#         #     continue
#         print("file_html: ", file_html)
#         try:
#             content = util.load_file_path(file_path)
#         except:
#             print(f"{file_path} is not existed!")
#             continue
#
#         try:
#             file_tree = ast.parse(content)
#             ana_py = ast_util.Fun_Analyzer()
#             ana_py.visit(file_tree)
#             # one_file_chained_comparison_code_list = []
#             #print("func number: ",file_html,len(ana_py.func_def_list))
#             dict_class = dict()
#             for tree, class_name in ana_py.func_def_list:
#                 # if 1:
#                 #print("come here",ast.unparse(tree))
#                 new_code_list = extract_compli_for_comprehension_logic.get_complicated_for_comprehen_code_list(tree, content, ["set()"], "add")
#                 for ind, (for_node, assign_node,remove_ass_flag) in enumerate(new_code_list):
#                     new_code_list[ind].append(
#                         transform_comprehension_set_compli_to_simple.transform(for_node, assign_node))
#                 #ode, assign_stmt, remove_ass_flag
#                 # for old_code,old_ass,remove_ass_flag,new_code in new_code_list:
#                 #     print("old_code,new_code: ",ast.unparse(old_code),ast.unparse(new_code))
#                 #
#                 # # one_file_chained_comparison_code_list.extend(get_complicated_for_comprehen_code_list(tree,content))
#                 ast_util.set_dict_class_code_list(tree,dict_class, class_name, new_code_list)
#
#                 # count+=len(one_file_chained_comparison_code_list)
#             dict_file[file_html] = dict_class
#             #print(dict_class)
#             # if one_file_chained_comparison_code_list:
#             #     one_repo_chained_comparison_code_list.append(
#             #         [one_file_chained_comparison_code_list, file_path, file_html])
#                 # print("one_file_truth_value_test_code_list: ",one_repo_chained_comparison_code_list)
#                     # break
#         except SyntaxError:
#             print("the file has syntax error")
#             continue
#         except ValueError:
#             traceback.print_exc()
#             print("the file has value error: ", file_html)#content,
#             continue
#         except:
#             traceback.print_exc()
#
#         # break
#     #print("count: ",count)
#     util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
#     print("save successfully! ", count,save_complicated_code_dir_pkl + repo_name)
if __name__ == '__main__':
    code = '''
# a=[]
# for i in range(4):
#     a.append(i)
# a=[]
for i in range(3):
    call(a)
a=set()
b=set()
for i in range(4):
    if i>3:
        for j in range(5):
            # i=i**2
            # a.append(j)
            b.add(i)
#     # elif i>4:
#     #     a.append(i)
# for i in range(4):
#      if i>3:
#          if i>3:
#              a.append(i)
#          else:
#               a.append(i)
# for far_node in tmp_far_nodes:
#     if not (mesh.has_node(far_node) and mesh.nodes[far_node].get('inpaint_id') == 1):
#         rmv_tmp_far_nodes.add(far_node)
'''
    #'''
    tree = ast.parse(code)
    code_list= extract_compli_for_comprehension_logic.get_complicated_for_comprehen_code_list(tree, code, ["set()"], "add")
    print("len: ",len(code_list))

    #'''
    # layer_node_list = []
    # ast_util.extract_ast_cur_layer_node(tree, layer_node_list)
    '''
    get comprehension node
    Block=> Assign;...; For
    For=>”for” ( For| If|If-else|body)
    If=>”if” (body| For| If| If-else)
    If-else=>”if” body “else” (body| If-else)
    body=> (Assign|Call)*;Append*
    '''
    # 对于
    # save_complicated_code_dir_pkl= util.data_root + "transform_complicate_to_simple_pkl/for_compre_set/"
    #
    # save_complicated_code_dir = util.data_root + "complicated_code_dir/for_comrephension_set_complicated_only_one_stmt/"
    # #'''
    # dict_repo_file_python = util.load_json(util.data_root, "python3_1000repos_files_info")
    # # repo_name_list=[]
    # # for repo_name in dict_repo_file_python:
    # #
    # #     repo_name_list.append(repo_name)
    # print("repo num: ", len(list(dict_repo_file_python.keys())))
    # count_complicated_code = 0
    # count_repo = 0
    # repo_list=[]
    # for ind, repo_name in enumerate(dict_repo_file_python):
    #     # if repo_name!="policy_sentry":#"pipreqs":#"spiderfoot":#"salt":#"Real-Time-Voice-Cloning":#"keras-bert":#"Legofy":
    #     #     continue
    #     # print("repo_name: ",repo_name)
    #     repo_list.append(repo_name)
    # # save_one_repo(repo_list[0])
    # import time
    # start_time=time.time()
    # '''
    # pool = newPool(nodes=30)
    # pool.map(save_one_repo, repo_list)  # [:3]sample_repo_url ,token_num_list[:1]
    # pool.close()
    # pool.join()
    # '''
    # files_num_list = []
    # star_num_list = []
    # contributor_num_list = []
    # count_repo, file_count, me_count, code_count = 0, 0, 0, 0
    # file_list = set([])
    # repo_code_num = dict()
    # result_compli_for_else_list = []
    # all_count_repo, all_file_count, all_me_count = 0, 0, 0
    # for file_name in os.listdir(save_complicated_code_dir_pkl):
    #     all_count_repo += 1
    #     repo_name = file_name[:-4]
    #     # files_num_list.append(repo_files_info[repo_name])
    #     # star_num_list.append(repo_star_info[repo_name])
    #     # contributor_num_list.append(repo_contributor_info[repo_name])
    #
    #     complicate_code = util.load_pkl(save_complicated_code_dir_pkl, repo_name)
    #
    #     repo_file_count, repo_me_count, repo_code_count, repo_all_file_count, repo_all_me_count = complicated_code_util.get_code_count(
    #         complicate_code)
    #     # for code_list, file_path, file_html in complicate_code:
    #     code_count += repo_code_count
    #     file_count += repo_file_count
    #     me_count += repo_me_count
    #     all_file_count += repo_all_file_count
    #     all_me_count += repo_all_me_count
    #     repo_exist = 0
    #     for file_html in complicate_code:
    #         for cl in complicate_code[file_html]:
    #             for me in complicate_code[file_html][cl]:
    #                 if complicate_code[file_html][cl][me]:
    #                     repo_exist = 1
    #                     for code in complicate_code[file_html][cl][me]:
    #                         # print("html: ", file_html, cl, me, ast.unparse(code1[0]))
    #                         # print("code1", code1)
    #                         #code_index_start_end_list.append([node,assign_stmt,remove_ass_flag])
    #                         result_compli_for_else_list.append(
    #                             [repo_name, file_html, cl, me, code[1].lineno,ast.unparse(code[1])+"\n"+ast.unparse(code[0]),ast.unparse(code[-1]),code[2]])
    #
    #         # print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")
    #     count_repo += repo_exist
    #
    # # a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
    # # print(a)
    # # print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
    # # print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
    # # print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
    # # print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
    # print("count: ", count_repo, code_count, file_count, me_count, all_count_repo, all_file_count, all_me_count)
    # import random
    #
    # random.shuffle(result_compli_for_else_list)
    # util.save_csv(util.data_root + "result_csv/for_comprehen_set.csv",
    #               result_compli_for_else_list[:400],
    #               ["repo_name", "file_html", "class_name", "me_name", "line_no", "old_code", "new_code","remove_ass_flag"])
    # end_time = time.time()
    # print("total time: ", end_time - start_time)
    # 1 156 2943 100 1 2990 40102 salt
    # 791 2010 103165 1291 800 121348 1192868
    # util.save_csv(util.data_root + "complicated_code_dir_pkl/for_compre_set.csv", result_compli_for_else_list,
    #               ["repo_name", "file_html", "class_name", "me_name", "for_code", "assign_code"])

# util.save_csv(util.data_root + "complicated_code_dir/for_comrephension_set_complicated_only_one_stmt.csv",
    #               result_compli_for_else_list, ["repo_name", "code1", "start_end_assign_ind", "file_html", "file_path"])
    #'''














import sys,ast,os

# sys.path.append("/mnt/zejun/smp/code1/")
# sys.path.append("/mnt/zejun/smp/code1/transform_c_s")


abs_path=os.path.abspath(os.path.dirname(__file__))
# print("abs_path: ",abs_path)
pack_path="/".join(abs_path.split("/")[:-1])
# print(pack_path)
sys.path.append(pack_path)
def transform_c_s_truth_value_test(node):
    def convert_func_var(node):
        # left_str = ast.unparse(node)
        func_name = ast.unparse(node.func)
        if func_name in ["bool"]:
            c = ast.Name(ast.unparse(node.args[0]))
            # c=ast.Call(  ast.Name('list'),args=[node.args[0]],keywords=[])

            return c#node.args[0]
        return node
    empty_set=["None", "False", "0", "0.0", "0j", "Decimal(0)", "Fraction(0, 1)", '', "()","[]", "{}", "dict()", "set()", "range(0)"]
    true_set=["True"]
    op = node.ops[0]
    comp_code_node=None
    left = node.left
    comparator = node.comparators[0]
    if isinstance(left, ast.Call):

        left_node = convert_func_var(left)
        comparator_node = comparator
        # print("come here call: ", ast.unparse(left),ast.unparse(left_node),ast.unparse(comparator_node))
    elif isinstance(comparator, ast.Call):
        left_node = left
        comparator_node = convert_func_var(comparator)
    else:
        left_node = left
        comparator_node = comparator
    left_str=ast.unparse(left_node)
    comparator_str=ast.unparse(comparator_node)
    if left_str in empty_set:
        if isinstance(op, (ast.Eq, ast.Is)):
            comp_code_node=ast.UnaryOp()
            comp_code_node.op=ast.Not()
            comp_code_node.operand=comparator_node
            complicate_code = "not " + comparator_str
        else:
            comp_code_node=comparator_node
            complicate_code = comparator_str
    elif comparator_str in empty_set:
        # print("come here")
        if isinstance(op, (ast.Eq, ast.Is)):
            comp_code_node = ast.UnaryOp()
            comp_code_node.op = ast.Not()
            comp_code_node.operand = left_node
            complicate_code = "not " + left_str
        else:
            comp_code_node = left_node
            complicate_code = left_str
    elif left_str in true_set:
        if isinstance(op, (ast.Eq, ast.Is)):
            comp_code_node=comparator
        else:
            comp_code_node = ast.UnaryOp()
            comp_code_node.op = ast.Not()
            comp_code_node.operand = comparator_node
    elif comparator_str in true_set:
        if isinstance(op, (ast.Eq, ast.Is)):
            comp_code_node=left
        else:
            comp_code_node = ast.UnaryOp()
            comp_code_node.op = ast.Not()
            comp_code_node.operand = left_node



    return comp_code_node
# def traverse(node, complicate_code, assign_str):
#     if isinstance(node, ast.For):
#         target = node.target
#         iter = node.iter
#         body = node.body[0]
#         complicate_code.append("for " + ast.unparse(target) + " in " + ast.unparse(iter) + " ")
#         traverse(body, complicate_code, assign_str)
#
#
#
#     elif isinstance(node, ast.If):
#         test = node.test
#         body = node.body[0]
#         if not node.orelse:
#
#             complicate_code.append("if " + ast.unparse(test) + " ")
#             traverse(body, complicate_code, assign_str)
#         else:  # 这个是一个赋值语句
#             if isinstance(body, ast.Expr) and isinstance(body.value, ast.Call):
#                 args_if = ast.unparse(body.value.args)
#
#             or_else = node.orelse[0]
#             if isinstance(or_else, ast.Expr) and isinstance(or_else.value, ast.Call):
#                 args_or_else = ast.unparse(or_else.value.args)
#
#                 assign_str.append(args_if + " if " + ast.unparse(test) + " else " + args_or_else + " ")
#             else:
#                 assign_str.append(args_if + " if " + ast.unparse(test) + " else ")
#                 traverse(or_else, complicate_code, assign_str)
#             # print(args_or_else)
#             # complicate_code.insert(0,args_if+" if " + ast.unparse(test) + " else "+args_or_else+" ")
#
#     elif isinstance(node, ast.Expr) and isinstance(node.value, ast.Call):
#         assign_str.append(ast.unparse(node.value.args) + " ")
        # complicate_code.insert(0,args_assign_expr)
if __name__ == '__main__':
    code='''
ri[0].methods == []
#a==0
#pipfile_path is None
#self.top_freq == 0.0
#config['offscreen_rendering'] is True
#len(np.sum(np.abs(image[..., 0] - image[..., 1]))) == 0
# 0 != len(np.sum(np.abs(image[..., 0] - image[..., 1]))) 

# inpaint_iter!= 0
'''
    empty_set=["None", "False", "0", "0.0", "0j", "Decimal(0)", "Fraction(0, 1)", '', "()","[]", "{}", "dict()", "set()", "range(0)"]

    tree= ast.parse(code)

    complicate_code=""
    def convert_func_var(node):
        left_str = ast.unparse(node)
        func_name = ast.unparse(node.func)
        if func_name in ["len", "bool"]:
            left_str = ast.unparse(node.args[0])
        return left_str


    for node in ast.walk(tree):
        '''
        这里的compare节点就是繁杂的代码   
        1. left right的node 解析转换为最终的字符串
        2. 获得为空的node，根据op进行转换
        '''
        if isinstance(node,ast.Compare):
            complicate_code=transform_c_s_truth_value_test(node)
            # op=node.ops[0]
            # left=node.left
            # comparator=node.comparators[0]
            # if isinstance(left,ast.Call):
            #     left_str=convert_func_var(left)
            #     comparator_str = ast.unparse(comparator)
            # elif isinstance(comparator,ast.Call):
            #     left_str=ast.unparse(left)
            #     comparator_str = convert_func_var(comparator)
            # else:
            #     left_str=ast.unparse(left)
            #     comparator_str=ast.unparse(comparator)
            #
            # if left_str in empty_set:
            #     if isinstance(op,(ast.Eq,ast.Is)):
            #         complicate_code="not "+comparator_str
            #     else:
            #         complicate_code = comparator_str
            # else:
            #     # print("come here")
            #     if isinstance(op,(ast.Eq,ast.Is)):
            #         complicate_code="not "+left_str
            #     else:
            #         complicate_code = left_str
    print(ast.unparse(complicate_code))


















import sys,ast,os
import traceback
abs_path=os.path.abspath(os.path.dirname(__file__))

# abs_path=os.getcwd()
# print("abs_path: ",abs_path)
pack_path="/".join(abs_path.split("/")[:-1])
# print(pack_path)
sys.path.append(pack_path)

import copy,time,complicated_code_util
import util
from sympy import *
from RefactoringIdioms.extract_simp_cmpl_data import ast_util

from RefactoringIdioms.transform_c_s import transform_var_unpack_call_compli_to_simple
from pathos.multiprocessing import ProcessingPool as newPool


def whether_add_end(slice_list, each_seq, arg_seq,step,new_arg_same_list, beg, end):
    if end == len(slice_list[1:]) and end > beg:
        new_arg_same_list.append(
            [each_seq[beg:end + 1], arg_seq[1][beg:end + 1], arg_seq[2], step]) # arg_list, ind_list, call node, step
def whether_add(each_seq, arg_seq,step,new_arg_same_list, beg, end):
    if end > beg:
        new_arg_same_list.append(
            [each_seq[beg:end + 1], arg_seq[1][beg:end + 1], arg_seq[2], step])
def get_func_call_by_args(tree):
    code_list = []
    arg_same_list=[]
    new_arg_same_list = []
    for node in ast.walk(tree):
        if isinstance(node, ast.Call):
            args_list = node.args
            var_set = []
            ind_list = []
            arg_one_list=[]
            # arg_same_list = []
            for ind_elts,e in enumerate(args_list):
                # print(args_list)

                '''
                找参数中连续使用同一变量的subscript
                '''

                if isinstance(e, ast.Subscript):
                    #print("arg: ",ast.unparse(e),ind_elts,var_set)
                    if isinstance(e.slice, ast.Slice):
                        if len(var_set) > 1:
                            arg_same_list.append([arg_one_list, ind_list,node])
                            # code_list.append([ast.unparse(node), ind_list])
                        var_set = []
                        ind_list = []
                        arg_one_list = []
                        continue
                    e_value = e.value
                    value_str=ast.unparse(e_value)
                    if not var_set:
                        var_set.append(e_value)
                        ind_list.append(ind_elts)
                        arg_one_list.append(e)
                    else:
                        if value_str==ast.unparse(var_set[-1]):
                            var_set.append(e_value)
                            ind_list.append(ind_elts)
                            arg_one_list.append(e)
                            if ind_elts==len(args_list)-1:
                                arg_same_list.append([arg_one_list, ind_list, node])

                        else:
                            if len(var_set)>1:
                                arg_same_list.append([arg_one_list,ind_list,node])
                                # code_list.append([ast.unparse(node), ind_list])
                            var_set=[e_value]
                            ind_list=[ind_elts]
                            arg_one_list=[e]
                            #print(">>>>>>>arg: ", ast.unparse(e), ind_elts,var_set)
                else:
                    if len(var_set) > 1:
                        arg_same_list.append([arg_one_list, ind_list,node])
                        # code_list.append([ast.unparse(node), ind_list])
                    var_set = []
                    ind_list = []
                    arg_one_list = []

    '''
    提取下标是等差数列的node_list
    '''
   # each arg_seq is arg_list where each ele is an arg, corresponding ind_list where each ele is the index of arg in the args, corresponding node
    for arg_seq in arg_same_list:
        # print("each_arg: ",arg_seq)
        each_seq=arg_seq[0]
        #print(each_seq)
        slice_list=[]

        for e_child in ast.walk(tree):
            if hasattr(e_child,'lineno'):

                if e_child.lineno<each_seq[0].lineno and isinstance(e_child,(ast.Assign,ast.AnnAssign)):
                    if isinstance(e_child,ast.Assign):
                        tar_list=e_child.targets
                    else:
                        tar_list=[e_child.target]
                    for tar in tar_list:
                        try:
                            if ast.unparse(tar)==ast.unparse(each_seq[0].value) \
                                and hasattr(e_child,"value") and ast.unparse(e_child.value)[0] in ["[","(","set("]:
                                break
                        except:
                            traceback.print_exc()
                            print("e_child: ",e_child)
                            print("e_child_str: ",ast.unparse(e_child))
                            continue


                    else:
                        continue
                    break
        else:
            continue
        if 1:

            # delete slice of arg is str type
            for arg in each_seq:
                slice=arg.slice
                slice_list.append(slice)
                # if isinstance(slice,ast.Constant) and not isinstance(slice.value, int):
                #     #print(">>>>>>>>it is str: ",arg_seq)
                #     continue

            #else:
                # determine whether slice list of arg 是等差数列
            step=None
            #for i in range(1,len(slice_list)-1):
            beg = 0
            end=0
            for ind,e_node in enumerate(slice_list[1:]):
                pre_var= slice_list[ind]
                pre_var_str = ast.unparse(slice_list[ind])
                e_str=ast.unparse(e_node)
                # determine whether slice of arg is int constant
                if 1:#pre_var_str.isdigit() and e_str.isdigit():
                    # print(">>>>>>>>it is constant: ",arg_seq)
                    if not step:
                        try:
                            step=str(sympify(e_str+'-('+pre_var_str+")"))
                        except:
                            step="None"
                        # print("step: ",step,ast.unparse(e_node),ast.unparse(pre_var))
                        # step=int(e_str)-int(pre_var_str)

                        if not step.isdigit() or (step.isdigit() and int(step)==0):
                            whether_add(each_seq, arg_seq, step, new_arg_same_list, beg, end)
                            step=None
                            end += 1
                            beg=end
                        else:
                            step=int(step)
                            end+=1
                            whether_add_end(slice_list, each_seq, arg_seq, step, new_arg_same_list, beg, end)

                    else:
                        # new_step=int(e_str)-int(pre_var_str)
                        try:
                            new_step=str(sympify(e_str+'-('+pre_var_str+")"))
                        except:
                            new_step="None"
                        # new_step = str(sympify(e_str + '-(' + pre_var_str + ")"))
                        # print("new_step: ", new_step, ast.unparse(e_node), ast.unparse(pre_var))
                        if new_step.isdigit() and int(new_step)==step:
                            end+=1
                            whether_add_end(slice_list, each_seq, arg_seq, step, new_arg_same_list, beg, end)
                        else:

                            whether_add(each_seq, arg_seq, step, new_arg_same_list, beg, end)
                            step=None
                            end+=1
                            beg = end
                    #print(">>>>>>>>it is not the same step: ", slice_list,pre_var,e,arg_seq)
                    # continue
            # else:# determine whether not int constant slice is the same step
            #     # e_node=ast.parse(e)
            #     #if is not constant, it must BinOp, and all nodes has the same step
            #     #print(">>>>>>>>it is not the constant index in slice: ", slice_list, pre_var, e, arg_seq)
            #     if isinstance(e_node,ast.BinOp) and ast.unparse(e_node.right).isdigit():
            #         if not step:
            #             #print(">>>>>>>>the step: ", ast.unparse(e_node), pre_var, e, arg_seq)
            #             if isinstance(pre_var, ast.BinOp) and ast.unparse(pre_var.right).isdigit():
            #
            #             step=int(ast.unparse(e_node.right))
            #             if step <= 0:
            #                 whether_add(each_seq, arg_seq, step, new_arg_same_list, beg, end)
            #                 step = None
            #                 end += 1
            #                 beg = end
            #             else:
            #                 end += 1
            #                 whether_add_end(slice_list, each_seq, arg_seq, step, new_arg_same_list, beg, end)
            #         else:
            #             new_step = int(ast.unparse(e_node.right))
            #             if new_step == step:
            #                 end += 1
            #                 whether_add_end(slice_list, each_seq, arg_seq, step, new_arg_same_list, beg, end)
            #             else:
            #
            #                 whether_add(each_seq, arg_seq, step, new_arg_same_list, beg, end)
            #                 step = None
            #                 end += 1
            #                 beg = end
            #                 # continue
            #     else:
            #         step = None
            #         whether_add(each_seq, arg_seq, step, new_arg_same_list, beg, end)
            #         end+=1
            #         beg = end


    #print(code_list)
    return new_arg_same_list
    # code_map=dict()
    # for e in new_arg_same_list:
    #     copy_e=e[-2]
    #     # copy_e = copy.deepcopy(e[-2])
    #     # e[-2] = ast.unparse(e[-2])
    #     if copy_e in code_map:
    #
    #
    #         code_map[copy_e].append(e)
    #     else:
    #         code_map[copy_e]=[e]
    #     code_list.append([e[1],e[2],e[3]])
    # # print(code_map)
    # return code_map
def save_repo_for_else_complicated(repo_name):
    start=time.time()
    count_complicated_code = 0
    print("come the repo: ", repo_name)
    one_repo_for_else_code_list = []

    dict_file=dict()
    # if os.path.exists(save_complicated_code_dir + repo_name + ".json"):
    #     print("the repo has been saved before")
    #     return None

    for ind,file_info in enumerate(dict_repo_file_python[repo_name]):

        file_path = file_info["file_path"]
        # if file_path!="/mnt/zejun/smp/data/python_repo_1000/VideoPose3D//run.py":
        #     continue
        file_html = file_info["file_html"]
        # if file_html!="https://github.com/quantumlib/OpenFermion/tree/master/src/openfermion/transforms/opconversions/bksf.py":#"https://github.com/flennerhag/mlens/tree/master/mlens/utils/id_train.py":#"https://github.com/google/budoux/tree/master/budoux/parser.py":#"https://github.com/django/django/tree/master/django/contrib/gis/gdal/raster/band.py":
        #     continue

        try:
            content = util.load_file_path(file_path)
        except:
            print(f"{file_path} is not existed!")
            continue

        # print("content: ",content)
        try:
            print("file_html: ", file_html)
            # print("come here")
            file_tree = ast.parse(content)
            ana_py = ast_util.Fun_Analyzer()
            ana_py.visit(file_tree)
            # print("ana_py.func_def_list ", ana_py.func_def_list)
            # dict_file["repo_name"]=repo_name
            dict_class=dict()
            for tree, class_name in ana_py.func_def_list:
                new_code_list=[]
                new_arg_same_list = get_func_call_by_args(tree)

                for ind, arg_info_list in enumerate(new_arg_same_list):
                    copy_arg_info_list=copy.deepcopy(arg_info_list)
                    star_node = transform_var_unpack_call_compli_to_simple.transform_var_unpack_call_each_args(
                        copy_arg_info_list)
                    new_code_list.append([arg_info_list, star_node])
                # print("new_code_list ", new_code_list)
                #
                # for old_code,new_code in new_code_list:
                #     print("old_code,new_code: ",old_code[1],ast.unparse(old_code[-2]),ast.unparse(new_code))
                ast_util.set_dict_class_code_list(tree, dict_class, class_name, new_code_list)
            dict_file[file_html]=dict_class
        except SyntaxError:
            print("the file has syntax error")
            continue
        except ValueError:
            print("the file has value error: ", content, file_html)
            continue
        # break
    end = time.time()
    #'''
    if dict_file:
        count_complicated_code = count_complicated_code + len(one_repo_for_else_code_list)
        # print("it exists for else complicated code1: ", len(one_repo_for_else_code_list))
        util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
        # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
        print(end-start," save successfully! ", save_complicated_code_dir_pkl + repo_name)
    else:
        print(end-start," the repo has no for else")
        util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
        # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
    #'''
    return count_complicated_code

if __name__ == '__main__':
    code='''
upfirdn2d_native(input, kernel, up, up, down, down, pad[0], pad[1], pad[0], pad[1])
ConvBNLayer(out_channels[1], out_channels[1], 3, 1, act='relu', name='fpn_up_g1_1')
sort_by_key_ir(ins[0], ins[1], outs[0], outs[1], outs[2], outs[3], axis, is_ascend)
a=axes.text( rect.xy[1],rect.xy[0], rect.xy[1], labels[I:],labels[i+1],labels[2],bbox=dict(facecolor=color, lw=0))
# a=axes.text( rect.xy[0],rect.xy[1], rect.xy[1], labels[I:],labels[i+1],labels[2],bbox=dict(facecolor=color, lw=0))
# b=func(a[i],a[i+1],c[1])
# b=func(a[i+1],a[i],a[i+1],c[1])
# b=func(a['1'],a[i],a[i+1],c[2],c[0],c[1])
b=func(a['1'],a[i],a[i+1],c[2],c['1'],c[0],c[1])
# b=func(a,b,c[1])
self.ds_opt_adam.adam_update_copy(self.opt_id, state['step'], group['lr'], beta1, beta2, group['eps'], group['weight_decay'], group['bias_correction'], p.data, p.grad.data, state['exp_avg'], state['exp_avg_sq'], fp16_param_groups[group_id][param_id].data)

'''
    '''
    # get_func_call_by_args(tree)
    tree = ast.parse(code1)
    code_list=get_func_call_by_args(tree)
    test_save_complicated_code_dir =util.data_root + "test_complicated_code_dir/"
    file_name="var_unpack_func_call_only_same_dengcha_subscript_complicated"

    # util.save_pkl(test_save_complicated_code_dir,file_name,code_list)
    for code1 in code_list:
        print("code1: ",code1)
    '''
    # save_complicated_code_dir = util.data_root + "complicated_code_dir/var_unpack_func_call_only_same_dengcha_subscript_complicated/"

    save_complicated_code_dir = util.data_root + "complicated_code_dir/var_unpack_func_call_only_same_dengcha_subscript_complicated_pkl/"
    save_complicated_code_dir_pkl= util.data_root + "transform_complicate_to_simple_pkl/var_unpack_call_star_complicated_strict/"

    dict_repo_file_python= util.load_json(util.data_root, "python3_1000repos_files_info")
    repo_name_list=[]
    for repo_name in dict_repo_file_python:
        # if repo_name!="OpenFermion":#"mlens":#"OpenFermion":#"budoux":#"django":
        #     continue
        # if os.path.exists(save_complicated_code_dir_pkl + repo_name + ".pkl"):
        #
        #     continue
        repo_name_list.append(repo_name)
    print("repo num: ", len(list(dict_repo_file_python.keys())),len(repo_name_list))
    # save_repo_for_else_complicated(repo_name_list[0])

    start_time = time.time()
    #'''
    pool = newPool(nodes=30)
    pool.map(save_repo_for_else_complicated, repo_name_list)  # [:3]sample_repo_url ,token_num_list[:1]
    pool.close()
    pool.join()
    #'''
    end_time = time.time()
    print("total time: ", end_time - start_time)

    files_num_list = []
    star_num_list = []
    contributor_num_list = []
    count_repo, file_count, me_count, code_count = 0, 0, 0, 0
    file_list = set([])
    repo_code_num = dict()
    result_compli_for_else_list = []
    all_count_repo, all_file_count, all_me_count = 0, 0, 0
    for file_name in os.listdir(save_complicated_code_dir_pkl):
        all_count_repo += 1
        repo_name = file_name[:-4]
        # files_num_list.append(repo_files_info[repo_name])
        # star_num_list.append(repo_star_info[repo_name])
        # contributor_num_list.append(repo_contributor_info[repo_name])

        complicate_code = util.load_pkl(save_complicated_code_dir_pkl, repo_name)

        repo_file_count, repo_me_count, repo_code_count, repo_all_file_count, repo_all_me_count = complicated_code_util.get_code_count(
            complicate_code)
        # for code_list, file_path, file_html in complicate_code:
        code_count += repo_code_count
        file_count += repo_file_count
        me_count += repo_me_count
        all_file_count += repo_all_file_count
        all_me_count += repo_all_me_count
        repo_exist = 0
        for file_html in complicate_code:
            for cl in complicate_code[file_html]:
                for me in complicate_code[file_html][cl]:
                    if complicate_code[file_html][cl][me]:
                        repo_exist = 1
                        for code in complicate_code[file_html][cl][me]:
                            pass
                            # print("html: ",file_html,cl,me,ast.unparse(code1[0]))
                            #                code_index_start_end_list.append([node,assign_stmt,node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str])

                            # result_compli_for_else_list.append(
                            #     [repo_name, file_html, cl, me, ast.unparse(code1[0][2]), ast.unparse(code1[1])])

            # print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")
        count_repo += repo_exist

    # a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
    # print(a)
    # print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
    # print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
    # print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
    # print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
    print("count: ", count_repo, code_count, file_count, me_count, all_count_repo, all_file_count, all_me_count)
    end_time = time.time()
    print("total time: ", end_time - start_time)
    # 1 156 2943 100 1 2990 40102 salt
    # 791 2010 103165 1291 800 121348 1192868
    # util.save_csv(util.data_root + "transform_complicate_to_simple_pkl/var_unpack_call_star_complicated.csv", result_compli_for_else_list,
    #               ["repo_name", "file_html", "class_name", "me_name", "for_code", "assign_code"])
    #


    # print("len all_files: ", len(all_files))
    # count=0
    # count_file=0
    # for file_name in os.listdir(save_complicated_code_dir):
    #     complicate_code = util.load_pkl(save_complicated_code_dir, file_name[:-4])
    #     for each_file in complicate_code:
    #         # for code_list, file_path,file_html in each_file:
    #         #
    #         #     print("count: ",code_list)
    #         code_map = each_file[0]
    #         file_path = each_file[1]
    #         file_html = each_file[2]
    #         count_file+=1
    #         for  node, item in code_map.items():
    #             count+=len(item)
    # print("count: ", count, count_file,len(os.listdir(save_complicated_code_dir)))
    #

    '''
    count=0
    result_compli_for_else_list=[]
    for file_name in os.listdir(save_complicated_code_dir):
        complicate_code=util.load_json(save_complicated_code_dir,file_name[:-5])
        for each_file in complicate_code:

            # for code_list, file_path,file_html in each_file:
            #
            #     print("count: ",code_list)
                code_list=each_file[0]
                file_path=each_file[1]
                file_html=each_file[2]
                count += len(code_list)
                # print("count: ", count)
                for code1 in code_list:
                    repo_name = file_html.split("/")[4]
                    result_compli_for_else_list.append(
                        [repo_name, code1[1], code1[0] ,code1[2], file_html, file_path])

            #     print("one code1: ",repo_name,code1,file_html,file_path)
            #     break
            # break
        # print("file: ",file_name)
        # break
    print("count: ",count,len(os.listdir(save_complicated_code_dir)))
    util.save_csv(util.data_root+"complicated_code_dir/var_unpack_func_call_only_same_dengcha_subscript_complicated.csv",result_compli_for_else_list,["repo_name","code1","start_end","file_html","file_path"])
    '''






import ast
import tokenize
import sys
# sys.path.append("../../..")
# sys.path.append("../../../../")
# sys.path.append("/mnt/zejun/smp/code1/")
# sys.path.append("/mnt/zejun/smp/code1/transform_c_s")
from RefactoringIdioms.extract_simp_cmpl_data import ast_util
from RefactoringIdioms.transform_c_s.transform_assign_multi_compli_to_simple import transform_multiple_assign
from tokenize import tokenize
from io import BytesIO
# from pathos.multiprocessing import ProcessingPool as newPool
import traceback
'''
input: 代码片段 code1
'''
def get_tokens(node):
    s = ast.unparse(node)
    g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
    tokens=set([])
    for toknum, child, _, _, _ in g:
        tokens.add(child)
    return tokens
class Visit_vars(ast.NodeVisitor):
    def __init__(self):
        self.vars = []
    def visit_Name(self, node) :
        self.vars.append(node)
    def visit_Attribute(self, node):
        self.vars.append(node)
    def visit_Subscript(self, node):
        self.vars.append(node)


# def visit_vars(target, list_vars):
#     # print(">>>>>>>target: ",target.__dict__)
#     if isinstance(target, (ast.Name)):
#         list_vars.append(ast.unparse(target))
#     elif  isinstance(target, ast.Subscript):
#         list_vars.append(ast.unparse(target))
#         visit_vars(target.value, list_vars)
#         visit_vars(target.slice, list_vars)
#     elif isinstance(target,ast.Attribute):
#         # print("attr: ",ast.unparse(target))
#         list_vars.append(ast.unparse(target))
#         visit_vars(target.value, list_vars)
#     else:
#         print("visit_vars else node: ", ast.unparse(target))
#         for e in ast.iter_child_nodes(target):
#             visit_vars(e, list_vars)
def visit_single_vars(target, list_vars):
    # print(">>>>>>>target: ",target.__dict__)
    if isinstance(target, (ast.Name)):
        list_vars.append(ast.unparse(target))
    elif  isinstance(target, ast.Subscript):
        list_vars.append(ast.unparse(target))
        if not isinstance(target.value,ast.Subscript):
            visit_single_vars(target.value, list_vars)
    elif isinstance(target,ast.Attribute):
        print("attr: ",ast.unparse(target))
        list_vars.append(ast.unparse(target))
        visit_single_vars(target.value, list_vars)
    else:
        print("visit_single_vars else node: ", ast.unparse(target))
        for e in ast.iter_child_nodes(target):
            print("visit_single_vars e: ",e,ast.unparse(e))
            visit_single_vars(e, list_vars)
def is_occur_body(same_value_var,body_list):

    for body in body_list:
        for e in ast.walk(body):
            if ast.unparse(e)==same_value_var:

                return 1

    return 0


def is_not_intersect_value_also_in_left(all_target_list, value):
    for tar in all_target_list:
        if tar == value:
            return 0
    return 1
def once_again(ass):
    a=[ass]
    target = ass.targets[0]
    value=ass.value
    write_vars = []
    visit_single_vars(target, write_vars)
    all_write_vars = [write_vars]
    all_target_list = [ast.unparse(target)]
    all_value_list=[ast.unparse(value)]
    return a,all_write_vars, all_target_list,all_value_list


def split_assignments_overlap_read_write(all_assign_list, all_body_list):
    def is_depend(pre_tar, next_tar):
        pre_tar_str = ast.unparse(pre_tar)
        for node in ast.walk(next_tar):
            if pre_tar_str == ast.unparse(node):
                return 1

        return 0

    all_assign_left_no_overlap_list = []
    for ind_ass, assign_list in enumerate(all_assign_list):
        # print("assign_list: ",assign_list)
        body_list = all_body_list[ind_ass]
        a = [assign_list[0]]
        for ind_e, ass in enumerate(assign_list[1:]):
            next_tar = ass.targets[0]
            for reverse_ind_e2, pre_ass in enumerate(a[::-1]):#
                ind_e2=len(a)-reverse_ind_e2-1
                pre_tar = pre_ass.targets[0]

                if is_depend(pre_tar, next_tar):

                    if len(a[:ind_e2+1]) > 1:
                        all_assign_left_no_overlap_list.append([a[:ind_e2+1], body_list])
                    # print("is_depend: ",a,a[ind_e2+1:])
                    a = a[ind_e2+1:]
                    a.append(ass)#[]
                    break
            else:
                a.append(ass)
        if len(a) > 1:
            all_assign_left_no_overlap_list.append([a, body_list])
    # print(all_assign_left_no_overlap_list)
    real_assign_list = []
    for ind_ass, (assign_list, body_list) in enumerate(all_assign_left_no_overlap_list):

        a = [assign_list[0]]
        overlap_flag=0
        for ind_e, ass in enumerate(assign_list[1:]):
            # print("whether add the ass: ",ast.unparse(ass))
            next_value = ass.value
            for reverse_ind_e2, pre_ass in enumerate(a[::-1]):
                ind_e2=len(a)-reverse_ind_e2-1
                pre_tar = pre_ass.targets[0]
                pre_value = pre_ass.value

                def intersect_var(pre_tar, next_value):
                    # pre_var_list = []
                    for node in ast.walk(next_value):
                        if ast.unparse(pre_tar)==ast.unparse(node):
                            return 1
                    for node in ast.walk(pre_tar):
                        if ast.unparse(node)==ast.unparse(next_value):
                            return 1
                    return 0
                    # visit_vars(pre_tar, pre_var_list)
                    # next_var_list = []
                    # visit_vars(next_value, next_var_list)
                    # if set(pre_var_list) & set(next_var_list):
                    #     # print("intersect: ",pre_var_list,next_var_list,set(pre_var_list) & set(next_var_list))
                    #     return 1
                    # return 0

                def value_is_in_target_list(pre_value, ass_list):
                    for ass in ass_list:
                        tar = ass.targets[0]
                        if ast.unparse(pre_value) == ast.unparse(tar):
                            return 1
                    return 0

                if intersect_var(pre_tar, next_value):
                    # print("intersect assign: ",ast.unparse(ass),ast.unparse(pre_ass))
                    if not is_depend(pre_tar, next_value) or is_occur_body(ast.unparse(pre_tar), body_list) or not value_is_in_target_list(pre_value, a[ind_e2 + 1:]):
                        # print("intersect assign: ", ast.unparse(ass), ast.unparse(pre_ass),a)
                        # if len(a[:ind_e2+1]) > 1:
                        #     real_assign_list.append(a[:ind_e2+1])
                        if len(a[:ind_e2+1]) > 1 and overlap_flag:
                            real_assign_list.append(a)
                        a = a[ind_e2+1:]
                        a.append(ass)
                        if ind_e!=len(assign_list[1:])-1:
                                overlap_flag=0
                        # print("intersect assign: ",a)
                        # overlap_flag = 1
                        break
                    else:
                        overlap_flag = 1

            else:
                a.append(ass)

        if len(a) > 1 and  overlap_flag:
            real_assign_list.append(a)

    return real_assign_list

'''
左边不存在数据依赖，(a1,a2) a1没有出现在a2中

'''
def split_assignments(all_assign_list,all_body_list):
    def is_depend(pre_tar, next_tar):
        pre_tar_str = ast.unparse(pre_tar)
        for node in ast.walk(next_tar):
            if pre_tar_str == ast.unparse(node):
                return 1

        # next_tar_str=ast.unparse(pre_tar)
        # for node in ast.walk(pre_tar):
        #     if next_tar_str == ast.unparse(node):
        #         return 1

        return 0
    def is_depend_str(pre_tar_str, next_tar):

        for node in ast.walk(next_tar):
            if pre_tar_str == ast.unparse(node):
                return 1
        return 0



    all_assign_left_no_overlap_list = []
    for ind_ass, assign_list in enumerate(all_assign_list):
        # print("assign_list: ",assign_list)
        body_list = all_body_list[ind_ass]
        a = [assign_list[0]]
        for ind_e, ass in enumerate(assign_list[1:]):
            next_tar = ass.targets[0]
            for reverse_ind_e2, pre_ass in enumerate(a[::-1]):#
                ind_e2=len(a)-reverse_ind_e2-1
                pre_tar = pre_ass.targets[0]

                if is_depend(pre_tar, next_tar):

                    if len(a[:ind_e2+1]) > 1:
                        all_assign_left_no_overlap_list.append([a[:ind_e2+1], body_list])
                    # print("is_depend: ",a,a[ind_e2+1:])
                    a = a[ind_e2+1:]
                    a.append(ass)#[]
                    break
            else:
                a.append(ass)
        if len(a) > 1:
            all_assign_left_no_overlap_list.append([a, body_list])
    # print(all_assign_left_no_overlap_list)
    # for ass,body_list in all_assign_left_no_overlap_list:
    #     print(">>>>>>>>>>>>>: ")
    #     for e_ass in ass:
    #         print("ass: ",ast.unparse(e_ass))

    real_assign_list = []
    for ind_ass, (assign_list, body_list) in enumerate(all_assign_left_no_overlap_list):

        a = [assign_list[0]]

        for ind_e, ass in enumerate(assign_list[1:]):
            # print("whether add the ass: ",ast.unparse(ass))
            next_value = ass.value
            for reverse_ind_e2, pre_ass in enumerate(a[::-1]):
                ind_e2=len(a)-reverse_ind_e2-1
                pre_tar = pre_ass.targets[0]
                pre_value = pre_ass.value

                def value_is_in_target_list(pre_value, ass_list):
                    for ass in ass_list:
                        tar = ass.targets[0]
                        if ast.unparse(pre_value) == ast.unparse(tar):
                            return 1
                    return 0
                def intersect_var(pre_tar, next_value):
                    # pre_var_list = []
                    pre_var_list=[]
                    for node in ast.walk(pre_tar):
                        pre_var_list.append(ast.unparse(node))
                        # if ast.unparse(pre_tar)==ast.unparse(node):
                        #     return 1
                    def visit_vars(target, list_vars, pre_var_list, pre_tar):
                        if isinstance(target, (ast.Name)):
                            list_vars.append(ast.unparse(target))
                        elif isinstance(target, ast.Subscript):
                            list_vars.append(ast.unparse(target))
                            if not(isinstance(target,type(pre_tar)) and ast.unparse(target.value) in pre_var_list):
                            #     list_vars.append( ast.unparse(target.value))
                            # else:

                                visit_vars(target.value, list_vars, pre_var_list, pre_tar)
                            visit_vars(target.slice, list_vars, pre_var_list, pre_tar)
                        elif isinstance(target, ast.Attribute):
                            # print("attr: ",ast.unparse(target))
                            list_vars.append(ast.unparse(target))
                            if not(isinstance(target, type(pre_tar)) and ast.unparse(target.value) in pre_var_list):
                            #     list_vars.append( ast.unparse(target.value))
                            # else:
                            # if not (isinstance(target, type(pre_tar)) and (ast.unparse(target.value) not in pre_var_list)):
                                visit_vars(target.value, list_vars, pre_var_list, pre_tar)

                            # visit_vars(target.value, list_vars)
                        elif isinstance(target, list):
                            for e in target:
                                visit_vars(e, list_vars, pre_var_list, pre_tar)
                        else:
                            # print("visit_vars else node: ", ast.unparse(target),type(target))
                            for e in ast.iter_child_nodes(target):
                                visit_vars(e, list_vars, pre_var_list, pre_tar)

                        pass
                    # print("next_value,pre_tar: ",ast.unparse(next_value),ast.unparse(pre_tar))
                    list_vars=[]
                    visit_vars(next_value, list_vars, pre_var_list, pre_tar)
                    if set(list_vars)&set(pre_var_list):
                        return 1
                    return 0
                    # for node in ast.walk(pre_tar):
                    #     if ast.unparse(node)==ast.unparse(next_value):
                    #         return 1
                    # return 0
                # list_vars=[]
                # visit_vars(pre_tar, list_vars)
                # print("list_vars: ", ast.unparse(pre_tar),ast.unparse(pre_ass),list_vars)
                # next_value_var_list=[]
                # print(">>>>>>>come here",ast.unparse(next_value),ast.unparse(ass))
                # visit_single_vars(next_value,next_value_var_list)
                # print("next_value_var_list: ",ast.unparse(next_value),ast.unparse(ass),next_value_var_list)
                if intersect_var(pre_tar, next_value) or intersect_var( next_value,pre_tar) :#set(list_vars)&set(next_value_var_list):#intersect_var(pre_tar, next_value):
                        # print("next_value: ", ast.unparse(next_value))
                        # print("pre target tokens: ",get_tokens(pre_tar))
                    # print("next_value tokens: ", get_tokens(next_value))
                    # vars=get_tokens(next_value)
                    # # vis_vars=Visit_vars()
                    # # vis_vars.visit(next_value)
                    # nex_value_dep_pre_tar=0
                    # # for var in vis_vars.vars:
                    # for var in vars:
                    #     print("var: ",var)
                    #     if  is_depend_str(var, pre_tar):
                    #         nex_value_dep_pre_tar=1
                    #         break
                    # print("intersect assign: ",ast.unparse(ass),ast.unparse(pre_ass))
                    # if (is_depend(pre_tar, next_value) or nex_value_dep_pre_tar):
                    #     print("come here depend: ",)
                        if not (is_depend(pre_tar, next_value) and not is_occur_body(ast.unparse(pre_tar), body_list) and value_is_in_target_list(pre_value, a[ind_e2 + 1:])):
                        #     a.append(ass)
                        # else:
                        #     print("intersect assign: ", ast.unparse(ass), ast.unparse(pre_ass),a,ind_e2+1)
                            if len(a[:ind_e2+1]) > 1:
                                real_assign_list.append(a[:ind_e2+1])
                            a = a[ind_e2+1:]
                            a.append(ass)
                            # print("the real a : ",[ast.unparse(e) for e in a])

                            # overlap_flag = 1
                            break
            else:
                a.append(ass)

        if len(a) > 1:
            real_assign_list.append(a)
    return real_assign_list

# def split_assignments(all_assign_list,all_body_list):
#     real_assign_list=[]
#     for ind_ass,assign_list in enumerate(all_assign_list):
#         body_list=all_body_list[ind_ass]
#
#         node=assign_list[0]
#
#         all_write_vars=[]
#         write_vars=[]
#         visit_vars(node.targets[0], write_vars)
#
#         all_write_vars.append(write_vars)
#         same_value_var=ast.unparse(node.targets[0])
#         pre_same_values=[ast.unparse(node.targets[0])]
#         a=[assign_list[0]]
#         next_read_vars=[]
#
#         '''
#         s.p=a
#         b=s # this cannot be simplified
#         '''
#         flag=0# whether assignment list contain intersect vars for writing vars and reading vars
#         for ind,ass in enumerate(assign_list[1:]):
#             value=ass.value
#             read_vars = []
#             visit_right_vars(value, read_vars)
#             intersect=set([])
#             for write_vars in all_write_vars:
#                 intersect|=set(read_vars)&set(write_vars)
#
#
#             #
#             if intersect:
#                 print("intersect: ",intersect, ast.unparse(ass))
#                 ind_list = []
#                 for e in intersect:
#
#                     for ind_write,write_var in enumerate(all_write_vars):
#                         if e in write_var:
#                             ind_list.append(ind_write)
#                 # print("ind_list",ind_list,ast.unparse(ass),a)
#                 new_write_vars = []
#                 visit_vars(ass.targets[0], new_write_vars)
#                 all_write_vars.append(new_write_vars)
#
#                 if len(set(ind_list))>1:
#                     print("multiple targets overlap: ", ind_list, ast.unparse(ass))
#                     if len(a) > 1 and flag:
#                         real_assign_list.append(a)
#                         a=[ass]
#
#                         if ind!=len(assign_list[1:])-1:
#                             flag = 0
#                     continue
#
#                 if set(pre_same_values) & intersect:
#
#                     print("same_value_var: ",same_value_var, intersect,ast.unparse(ass))
#                     # is_occur=0
#                     for same_value_var in pre_same_values:
#                         if is_occur_body(same_value_var,body_list):
#
#
#                             if len(a)>1 and flag:
#                                 real_assign_list.append(a)
#                             a=[ass]
#                             if ind!=len(assign_list[1:])-1:
#                                 flag = 0
#                             is_occur=1
#                             break
#                     else:
#                         flag = 1
#                         a.append(ass)
#
#
#                 else:
#
#                     if len(a) > 1 and flag:
#                         real_assign_list.append(a)
#
#                     a = [ass]
#                     if ind != len(assign_list[1:]) - 1:
#                         flag = 0
#
#                     # continue
#                 pre_same_values.append(ast.unparse(ass.targets[0]))
#
#             else:
#                 print("it is not intersect")
#                 new_write_vars = []
#                 visit_vars(ass.targets[0], new_write_vars)
#                 all_write_vars.append(new_write_vars)
#                 pre_same_values.append(ast.unparse(ass.targets[0]))
#                 a.append(ass)
#         if len(a)>1 and flag:
#             real_assign_list.append(a)
#
#
#
#     return real_assign_list
#     pass
def transform_multiple_assign_code(content):
    code_pair_list = []
    # print(content)
    try:

        file_tree = ast.parse(content)
        ana_py = ast_util.Fun_Analyzer()
        ana_py.visit(file_tree)

        for tree, class_name in ana_py.func_def_list:
            try:
                me_name=tree.name
            except:
                me_name=""
            all_assign_list, all_body_list = get_multiple_assign(tree)
            # assign_list = split_assignments_overlap_read_write(all_assign_list, all_body_list)
            assign_list = split_assignments(all_assign_list, all_body_list)
            # print(">>>>>>>>>>>>>>come here: ",assign_list)


            for ind_ass, each_assign_list in enumerate(assign_list):
                # print(">>>>>>ind_ass: ", ind_ass)
                each_assign_list_str = "\n".join([ast.unparse(e_ass) for e_ass in each_assign_list])
                # each_assign_list_str_list = [ast.unparse(e_ass) for e_ass in each_assign_list]
                # old_node_content = ast.get_source_segment(content, old_node)
                each_assign_list_str_list = [ ast.get_source_segment(content, e_ass) for e_ass in each_assign_list]
                new_code = transform_multiple_assign(each_assign_list)

                new_code_list=[new_code]+[ '' for e_ass in each_assign_list[:-1]]
                line_list = [[(e_ass.lineno,e_ass.col_offset), (e_ass.end_lineno,e_ass.end_col_offset)] for e_ass in each_assign_list]

                # print("new_code_list: ",new_code_list)
                code_pair_list.append([class_name, me_name, each_assign_list_str_list, new_code_list, line_list])

                # line_list =[[e_ass.lineno, e_ass.end_lineno] for e_ass in each_assign_list]
                # # line_list.append([[e_ass.lineno, e_ass.end_lineno] for e_ass in each_assign_list])
                # new_code = transform_multiple_assign(each_assign_list)
                #
                # # new_file_content = replace_file_content_ass(content, each_assign_list, new_code)
                # # assign_list[ind_ass].append(new_code)
                # # complic_code_me_info_dir_pkl = util.data_root + "complic_code_me_info_dir_pkl/each_idiom_type_all_methods/multi_ass/"  # for_else
                # # util.save_file(complic_code_me_info_dir_pkl, "test"+str(ind_ass), new_file_content, ".txt", "w")
                # # util.save_file(complic_code_me_info_dir_pkl, "test_old"+str(ind_ass), content, ".txt", "w")
                #
                # code_pair_list.append([class_name,me_name,each_assign_list_str_list, [new_code],line_list])

        return code_pair_list

    except:
        traceback.print_exc()
        return code_pair_list

def transform_part_multiple_assign_code(content):
    code_pair_list = []
    # print(content)
    try:

        tree = ast.parse(content)
        if 1:
            try:
                me_name=tree.name
            except:
                me_name=""
            all_assign_list, all_body_list = get_multiple_assign(tree)
            # assign_list = split_assignments_overlap_read_write(all_assign_list, all_body_list)
            assign_list = split_assignments(all_assign_list, all_body_list)
            # print(">>>>>>>>>>>>>>come here: ",assign_list)


            for ind_ass, each_assign_list in enumerate(assign_list):
                # print(">>>>>>ind_ass: ", ind_ass)
                each_assign_list_str = "\n".join([ast.unparse(e_ass) for e_ass in each_assign_list])
                # each_assign_list_str_list = [ast.unparse(e_ass) for e_ass in each_assign_list]
                # old_node_content = ast.get_source_segment(content, old_node)
                each_assign_list_str_list = [ ast.get_source_segment(content, e_ass) for e_ass in each_assign_list]
                new_code = transform_multiple_assign(each_assign_list)

                new_code_list=[new_code]+[ '' for e_ass in each_assign_list[:-1]]
                line_list = [[(e_ass.lineno,e_ass.col_offset), (e_ass.end_lineno,e_ass.end_col_offset)] for e_ass in each_assign_list]

                # print("new_code_list: ",new_code_list)
                code_pair_list.append(['', me_name, each_assign_list_str_list, new_code_list, line_list])

                # line_list =[[e_ass.lineno, e_ass.end_lineno] for e_ass in each_assign_list]
                # # line_list.append([[e_ass.lineno, e_ass.end_lineno] for e_ass in each_assign_list])
                # new_code = transform_multiple_assign(each_assign_list)
                #
                # # new_file_content = replace_file_content_ass(content, each_assign_list, new_code)
                # # assign_list[ind_ass].append(new_code)
                # # complic_code_me_info_dir_pkl = util.data_root + "complic_code_me_info_dir_pkl/each_idiom_type_all_methods/multi_ass/"  # for_else
                # # util.save_file(complic_code_me_info_dir_pkl, "test"+str(ind_ass), new_file_content, ".txt", "w")
                # # util.save_file(complic_code_me_info_dir_pkl, "test_old"+str(ind_ass), content, ".txt", "w")
                #
                # code_pair_list.append([class_name,me_name,each_assign_list_str_list, [new_code],line_list])

        return code_pair_list

    except:
        # traceback.print_exc()
        return code_pair_list
def get_multiple_assign(tree):

    assign_list = []
    body_list = []
    for e_trr in ast.walk(tree):
        if hasattr(e_trr, "body") :
            body = e_trr.body
            if not isinstance(body,list):
                continue
            # print("body: ",ast.unparse(e_trr),ast.unparse(body))

            a = []

            for ind_node, node in enumerate(body):
                if isinstance(node, ast.Assign):
                    targets = node.targets
                    count = 0
                    for e in targets:
                        count += ast_util.get_basic_count(e)
                    if count > 1:
                        if len(a) > 1:
                            assign_list.append(a)
                            if ind_node + 1 <= len(body) - 1:
                                body_list.append(body[ind_node+1:])
                            else:
                                body_list.append([])
                        a = []
                        continue
                    else:
                        a.append(node)
                        if ind_node == len(body) - 1 and len(a) > 1:
                            assign_list.append(a)
                            body_list.append([])
                else:
                    if len(a) > 1:
                        assign_list.append(a)
                        if ind_node<=len(body) - 1:
                            body_list.append(body[ind_node:])
                        else:
                            body_list.append([])
                    a=[]
        if hasattr(e_trr, "orelse") :
            body = e_trr.orelse
            if not isinstance(body, list):
                continue
            # print("body: ",ast.unparse(e_trr),ast.unparse(body))

            a = []

            for ind_node, node in enumerate(body):
                if isinstance(node, ast.Assign):
                    targets = node.targets
                    count = 0
                    for e in targets:
                        count += ast_util.get_basic_count(e)
                    if count > 1:
                        if len(a) > 1:
                            assign_list.append(a)
                            if ind_node + 1 <= len(body) - 1:
                                body_list.append(body[ind_node + 1:])
                            else:
                                body_list.append([])
                        a = []
                        continue
                    else:
                        a.append(node)
                        if ind_node == len(body) - 1 and len(a) > 1:
                            assign_list.append(a)
                            body_list.append([])
                else:
                    if len(a) > 1:
                        assign_list.append(a)
                        if ind_node <= len(body) - 1:
                            body_list.append(body[ind_node:])
                        else:
                            body_list.append([])
                    a = []

    return assign_list,body_list
# def save_repo_for_else_complicated(repo_name):
#     start=time.time()
#     count_complicated_code = 0
#     print("come the repo: ", repo_name)
#     dict_file=dict()
#
#
#     for ind,file_info in enumerate(dict_repo_file_python[repo_name]):
#         file_path = file_info["file_path"]
#         # if file_path!="/mnt/zejun/smp/data/python_repo_1000/VideoPose3D//run.py":
#         #     continue
#         file_html = file_info["file_html"]
#         # print(file_html)
#         if "bm_scimark.py" not in file_html:
#             continue
#         # if file_html!="https://github.com/devbisme/skidl/tree/master/skidl/netlist_to_skidl.py":#"https://github.com/akfamily/akshare/tree/master/akshare/cost/cost_living.py":##"https://github.com/mitmproxy/mitmproxy/tree/master/mitmproxy/net/http/url.py":#"https://github.com/localstack/localstack/tree/master/localstack/utils/aws/aws_responses.py":#"https://github.com/encode/django-rest-framework/tree/master/rest_framework/utils/urls.py":#"https://github.com/spulec/moto/tree/master/moto/utilities/docker_utilities.py":#"https://github.com/mitmproxy/mitmproxy/tree/master/mitmproxy/net/http/url.py":#"https://github.com/docker/docker-py/tree/master/docker/models/services.py":#"https://github.com/mitmproxy/mitmproxy/tree/master/mitmproxy/net/http/url.py":#"https://github.com/facebookresearch/Detectron/tree/master/detectron/modeling/detector.py":#"https://github.com/chainer/chainer/tree/master/chainer/functions/normalization/group_normalization.py":
#         #     continue
#         print(file_html)
#         try:
#             content = util.load_file_path(file_path)
#         except:
#             print(f"{file_path} is not existed!")
#             continue
#
#         # print("content: ",content)
#         try:
#             file_tree = ast.parse(content)
#             ana_py = ast_util.Fun_Analyzer()
#             ana_py.visit(file_tree)
#             dict_class=dict()
#             for tree, class_name in ana_py.func_def_list:
#
#                 all_assign_list, all_body_list = get_multiple_assign(tree)
#                 # assign_list = split_assignments_overlap_read_write(all_assign_list, all_body_list)
#                 assign_list = split_assignments(all_assign_list, all_body_list)
#                 # print(">>>>>>>>>>>>>>come here: ",assign_list)
#                 new_code_list = []
#                 for ind_ass, each_assign_list in enumerate(assign_list):
#                     # print(">>>>>>ind_ass: ", ind_ass)
#                     for e_ass in each_assign_list:
#                         print("e_ass: ",e_ass,ast.unparse(e_ass))
#                     new_code = transform_multiple_assign(each_assign_list)
#                     print("new_code: ",new_code)
#                     # new_file_content = replace_file_content_ass(content, each_assign_list, new_code)
#                     # assign_list[ind_ass].append(new_code)
#                     # complic_code_me_info_dir_pkl = util.data_root + "complic_code_me_info_dir_pkl/each_idiom_type_all_methods/multi_ass/"  # for_else
#                     # util.save_file(complic_code_me_info_dir_pkl, "test"+str(ind_ass), new_file_content, ".txt", "w")
#                     # util.save_file(complic_code_me_info_dir_pkl, "test_old"+str(ind_ass), content, ".txt", "w")
#
#                     new_code_list.append([each_assign_list, new_code])
#                 if new_code_list:
#                     ast_util.set_dict_class_code_list(tree, dict_class, class_name, new_code_list)
#
#
#
#             dict_file[file_html]=dict_class
#
#             # if code_list:
#             #     one_repo_for_else_code_list.append([code_list, file_path, file_html])
#                 # if len(one_repo_for_else_code_list)>2:
#                 #     break
#
#         except SyntaxError:
#             print("the file has syntax error")
#             continue
#         except ValueError:
#             traceback.print_exc()
#             print("the file has value error: ", file_html)  # content,
#             continue
#         except:
#             traceback.print_exc()
#         # break
#     end = time.time()
#     #'''
#     if dict_file:
#         # count_complicated_code = count_complicated_code + len(one_repo_for_else_code_list)
#         print("it exists for else complicated code1")
#         # util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
#         # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
#         print(end-start," save successfully! ", save_complicated_code_dir_pkl + repo_name)
#     else:
#         print(end-start," the repo has no for else")
#         # util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
#         # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
#     #'''
#     return count_complicated_code

if __name__ == '__main__':
    code='''
self.a=1
b=self.call(2)
url = urllib.parse.urlsplit(url)

url = list(url)

url[3] = urllib.parse.quote(url[3])

url = urllib.parse.urlunsplit(url)

response = BaseResponse()

response.querystring = qs_data

result = response._get_multi_param(top_level_attribute, skip_result_conversion=True)
(scheme, netloc, path, query, fragment) = parse.urlsplit(force_str(url))

query_dict = parse.parse_qs(query, keep_blank_values=True)

query_dict[force_str(key)] = [force_str(val)]

query = parse.urlencode(sorted(query_dict.items()), doseq=True)
a[0]=1
b=a[1]

# code_obj = BytesIO(blobdata)
# self._tar = tarfile.open(fileobj=code_obj, mode='r:gz')
# info_str = self._tar.extractfile('INFO').read().decode('utf-8')
# self._info = json.loads(info_str)

# serializer_class_store = self.serializer_class
# self.serializer_class = self.copy_return_serializer_class
# ret = self.get_serializer(*args, **kwargs)
# self.serializer_class = serializer_class_store

canvas = self.canvas
self.canvas = self.fbo
ret = super(FboFloatLayout, self).add_widget(*args, **kwargs)
self.canvas = canvas

# if_return_trans = module.return_transform
# if_return_label = module.return_label
# module.return_transform = False
# module.return_label = False
# out = module(*args, **kwargs)
# module.return_transform = if_return_trans
# module.return_label = if_return_label

self._load_model = self.model is not None
__model = self.model
self.model = None
path = super().save(path=path, verbose=verbose)
self.model = __model

e = throttling_mod_func(d, e)#https://github.com/pytube/pytube/blob/f06e0710dcf5089e582487fee94f7bb0afbf7ba9/pytube/cipher.py#L606-L611
f = d[0]
d[0] = d[e]
d[e] = f

container_spec = ContainerSpec(**container_spec_kwargs)

task_template_kwargs['container_spec'] = container_spec

create_kwargs['task_template'] = TaskTemplate(**task_template_kwargs)
#https://github.com/ansible/ansible/blob/a71ba817b0f7d75ff848000226ed5241c791f476/test/support/integration/plugins/module_utils/network/common/utils.py#L591-L594
# params = basic._ANSIBLE_ARGS
# basic._ANSIBLE_ARGS = to_bytes(json.dumps({'ANSIBLE_MODULE_ARGS': data}))
# validated_data = basic.AnsibleModule(spec).params
# basic._ANSIBLE_ARGS = params

# tmp_real = data[ii]
# tmp_imag = data[ii + 1]
# data[ii] = data[jj]
# data[ii + 1] = data[jj + 1]
# data[jj] = tmp_real
# data[jj + 1] = tmp_imag

# hidden_states = residual + hidden_states
# residual = hidden_states
# hidden_states = self.layer_norm2(hidden_states)
# integral_w = paddle.concat(result_w, axis=0)

# model_b = SimpleEmbedding(vocab_size, hidden_size, integral_w)
# 
# optimizer_a = paddle.optimizer.SGD(learning_rate=0.001,
#                                    parameters=model_a.parameters())
# 
# optimizer_b = paddle.optimizer.SGD(learning_rate=0.001,
#                                    parameters=model_b.parameters())
# for _ in range(5):
#             output_a = model_a(input_data)
#             func(a)
#             output_b = model_b(input_data)
           
# a=1
# b=1
# c=b
# src_polys = np.array(src_polys)
# src_polys[:, :, 0] = np.clip(src_polys[:, :, 0], 0, w)
# src_polys[:, :, 1] = np.clip(src_polys[:, :, 1], 0, h)
# data['image'] = src_img
# data['polys'] = src_polys
# data['ignore_tags'] = np.array(src_ignores)
# exc = vim.fault.NoPermission()
# exc.privilegeId = "Fake privilege"
# self.mock_prof_mgr.QueryProfile = MagicMock(side_effect=exc)
# with self.assertRaises(VMwareApiError) as excinfo:
#     salt.utils.pbm.get_storage_policies(self.mock_prof_mgr)
# b = np.arange(12).reshape((3, 4))
# a = da.from_array(c, 1)
# c=1
# _assert(a, b)
# _assert(a, b, 0)
# _assert(a, b, 1)
# _assert(a, b, -1)

# b = np.arange(8).reshape((2, 2, 2))
# a = da.from_array(b, 2)
# lambda x: list(x)[0]
# a=2
# if label_names:
#     # a=2
#     num_tensors_in_label = len(label_names)
# else:
#     num_tensors_in_label = int(has_labels)
# #a,b=1,2
# eeeeee,a=1,2
# b=c.b+1
# for i in range(1):
#         c=1
#         d=2
#         e=3
#         for i in range(2):
#             f=1
#             r=0
#             c=2
# if self . get_conf_value ( 'show' , header = header ) == [ ] :
#     pass
# elif stats_grab == { } :
#     pass
# assert policy . remember ( pretend . stub ( ) , pretend . stub ( ) ) == 0
# d=0
# a.b=func()
# 
# for backend in ["auto", "cv2", "skimage"]:
#     with self.subTest(backend=backend):
#         aug = iaa.Affine(rotate=45, fit_output=True,
#                          backend=backend)
#         _labels, nb_labels = skimage.morphology.label(
#             img_aug > 240, return_num=True, connectivity=2)
#         img = np.zeros((10, 10), dtype=np.uint8)
#         img[0:2, 0:2] = 255
#         img[-2:, 0:2] = 255
#         img[0:2, -2:] = 255
#         img[-2:, -2:] = 255
# 
#         img_aug = aug.augment_image(img)
# 
#         _labels, nb_labels = skimage.morphology.label(
#             img_aug > 240, return_num=True, connectivity=2)
#         assert nb_labels == 4
# while a!=[]:
#     w=1
#     d=w+1
#     for i in range(1):
#         c=1
#         d=2
#         e=3
#         for i in range(2):
#             f=1
#             r=0
#             c=2
#         a=1
#         b=func()
#         c=2
#     pass
# class TPUEstimatorSpec(model_fn_lib._TPUEstimatorSpec):
#     def __new__(cls,
#               mode,
#               predictions=None,
#               loss=None,
#               train_op=None,
#               eval_metrics=None,
#               export_outputs=None,
#               scaffold_fn=None,
#               host_call=None,
#               training_hooks=None,
#               evaluation_hooks=None,
#               prediction_hooks=None):
#         """Creates a validated `TPUEstimatorSpec` instance."""
#         host_calls = {}
#         if eval_metrics is not None:
#           host_calls['eval_metrics'] = eval_metrics
#         if host_call is not None:
#           host_calls['host_call'] = host_call
#         _OutfeedHostCall.validate(host_calls)
#     
#         training_hooks = list(training_hooks or [])
#         evaluation_hooks = list(evaluation_hooks or [])
#         prediction_hooks = list(prediction_hooks or [])
#     lambda x: list(x)[0]
#         for hook in training_hooks + evaluation_hooks + prediction_hooks:
#           if not isinstance(hook, session_run_hook.SessionRunHook):
#             raise TypeError(
#                 'All hooks must be SessionRunHook instances, given: {}'.format(
#                     hook))
#     
#         return super(TPUEstimatorSpec, cls).__new__(
#             cls,
#             mode=mode,
#             predictions=predictions,
#             loss=loss,
#             train_op=train_op,
#             eval_metrics=eval_metrics,
#             export_outputs=export_outputs,
#             scaffold_fn=scaffold_fn,
#             host_call=host_call,
#             training_hooks=training_hooks,
#             evaluation_hooks=evaluation_hooks,
#             prediction_hooks=prediction_hooks)
'''
    '''
    tree = ast.parse(code1)
    code_list=[]
    all_assign_list,all_body_list = get_multiple_assign(tree)
    print(all_assign_list,all_body_list)
    # real_ass_list=split_assignments_overlap_read_write(all_assign_list, all_body_list)
    real_ass_list=split_assignments(all_assign_list, all_body_list)
    for ind_ass, each_assign_list in enumerate(real_ass_list):
        a = []
        for node in each_assign_list:
            a.append(ast.unparse(node))
            # print("come here: ", a.append(ast.unparse(node)))
        if a:
            # print(a)
            code_list.append(a)
    for code1 in code_list:
        print("code1: ",code1)
    print(">>>>>end")
    '''

    '''
    save_complicated_code_dir = util.data_root + "complicated_code_dir/multip_assign_complicated_single_target/"
    
    dict_repo_file_python = util.load_json(util.data_root, "python3_repos_files_info")

    count_complicated_code = 0
    for ind, repo_name in enumerate(dict_repo_file_python):
        #print(ind)
        
        one_repo_mult_assign_code_list = []
        for file_info in dict_repo_file_python[repo_name]:
            file_path = file_info["file_path"]
            file_html = file_info["file_html"]
            content = util.load_file(file_path)
            try:
                # if 1:
                tree = ast.parse(content)
                layer_node_list = []
                ast_util.extract_ast_cur_layer_node(tree,layer_node_list)
                all_assign_list = get_multiple_assign(layer_node_list)
                all_assign_str_list =[]
                for assign_list in all_assign_list:
                    for ind_ass, each_assign_list in enumerate(assign_list):
                        a=[]
                        for node in each_assign_list:
                            a.append(ast.unparse(node))
                            # print("come here: ", a.append(ast.unparse(node)))
                        if a:
                            all_assign_str_list.append(a)
                
                count_complicated_code += len(all_assign_list)
                if all_assign_list:
                    one_repo_mult_assign_code_list.append(
                        [all_assign_str_list, file_path, file_html])
                    # print("one_file_truth_value_test_code_list: ",one_file_truth_value_test_code_list)
                    # break
            except SyntaxError:
                print("the file has syntax error")
                continue
        if one_repo_mult_assign_code_list:
            # print("it exists truth value test complicated code1: ", len(one_repo_for_else_code_list))
            util.save_json(save_complicated_code_dir, repo_name, one_repo_mult_assign_code_list)
        
    print("count_complicated_code: ", ind, count_complicated_code)
    '''
    '''
    tree=ast.parse(code1)
    all_assign_list, all_body_list = get_multiple_assign(tree)
    # print("all_assign_list: ",all_assign_list)
    # assign_list = split_assignments_overlap_read_write(all_assign_list, all_body_list)
    assign_list = split_assignments(all_assign_list, all_body_list)
    # print("come here")
    new_code_list = []
    for ind_ass, each_assign_list in enumerate(assign_list):
        print(">>>>>>ind_ass: ", ind_ass)
        for e_ass in each_assign_list:
            print("e_ass: ", e_ass, ast.unparse(e_ass))
        new_code = transform_multiple_assign(each_assign_list)
        print("new_code: ", new_code)
    '''

    # ana_py = Analyzer()
    # ana_py.visit(file_tree)
    # print("fun number: ",len(ana_py.func_def_list))
    # beg_time=time.time()
    # save_complicated_code_dir= util.data_root + "complicated_code_dir/multip_assign_complicated_json/"
    # save_complicated_code_dir_pkl= util.data_root + "transform_complicate_to_simple_pkl/multip_assign_complicated/"
    # # save_complicated_code_dir_pkl=util.data_root +"transform_complicate_to_simple_pkl/var_unpack_for_target_complicated/"
    #
    # # dict_repo_file_python=util.load_json(util.data_root, "python3_repos_files_info" )
    # dict_repo_file_python= util.load_json(util.data_root, "python3_1000repos_files_info")
    #
    # repo_name_list=[]
    # for repo_name in dict_repo_file_python:
    #
    #     # if os.path.exists(save_complicated_code_dir_pkl + repo_name + ".pkl"):
    #     #     continue
    #     if repo_name!="pyperformance":#"pytube":#"skidl":#"akshare":#"mitmproxy":#"localstack":#"django-rest-framework":#"moto":#"docker-py":#"cloud-custodian":#"mitmproxy":#"mitmproxy":
    #         continue
    #
    #     repo_name_list.append(repo_name)
    # print("repo_name_list:",repo_name_list)
    # save_repo_for_else_complicated(repo_name_list[0])
    '''
    count_complicated_code=0
    for ind,repo_name in enumerate(dict_repo_file_python):
        one_repo_truth_value_test_code_list = []
        for file_info in dict_repo_file_python[repo_name]:
            file_path = file_info["file_path"]
            file_html = file_info["file_html"]
            content = util.load_file(file_path)
            try:
            #if 1:
                one_file_truth_value_test_code_list=get_truth_value_test(content)
                count_complicated_code+=len(one_file_truth_value_test_code_list)
                if one_file_truth_value_test_code_list:
                    one_repo_truth_value_test_code_list.append([one_file_truth_value_test_code_list, file_path, file_html])
                    # print("one_file_truth_value_test_code_list: ",one_file_truth_value_test_code_list)
                    # break
            except SyntaxError:
                print("the file has syntax error")
                continue
            # break
        if one_repo_truth_value_test_code_list:
            # print("it exists truth value test complicated code1: ", len(one_repo_for_else_code_list))
            util.save_json(save_complicated_code_dir, repo_name, one_repo_truth_value_test_code_list)

        # break
    # print()


        # break
    print("count_complicated_code: ",ind,count_complicated_code)
    
    '''
    '''
    pool = newPool(nodes=30)
    pool.map(save_repo_for_else_complicated, repo_name_list)  # [:3]sample_repo_url ,token_num_list[:1]
    pool.close()
    pool.join()
    end_time = time.time()
    print("time of extracting all complicated code1: ", end_time-beg_time,len(repo_name_list))
    '''
    #'''

    # files_num_list = []
    # star_num_list = []
    # contributor_num_list = []
    # count_repo, file_count, me_count, code_count = 0, 0, 0, 0
    # file_list = set([])
    # repo_code_num = dict()
    # result_compli_for_else_list = []
    # all_count_repo, all_file_count, all_me_count = 0, 0, 0
    # for file_name in os.listdir(save_complicated_code_dir_pkl):
    #     all_count_repo += 1
    #     repo_name = file_name[:-4]
    #     # files_num_list.append(repo_files_info[repo_name])
    #     # star_num_list.append(repo_star_info[repo_name])
    #     # contributor_num_list.append(repo_contributor_info[repo_name])
    #
    #     complicate_code = util.load_pkl(save_complicated_code_dir_pkl, repo_name)
    #
    #     repo_file_count, repo_me_count, repo_code_count, repo_all_file_count, repo_all_me_count = complicated_code_util.get_code_count(
    #         complicate_code)
    #     # for code_list, file_path, file_html in complicate_code:
    #     code_count += repo_code_count
    #     file_count += repo_file_count
    #     me_count += repo_me_count
    #     all_file_count += repo_all_file_count
    #     all_me_count += repo_all_me_count
    #     repo_exist = 0
    #     for file_html in complicate_code:
    #         for cl in complicate_code[file_html]:
    #             for me in complicate_code[file_html][cl]:
    #                 if complicate_code[file_html][cl][me]:
    #                     repo_exist = 1
    #                     for code in complicate_code[file_html][cl][me]:
    #                         # print("html: ",file_html,cl,me,ast.unparse(code1[0]))
    #                         #                code_index_start_end_list.append([node,assign_stmt,node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str])
    #                         ass_str=[]
    #                         for ass in code[0]:
    #                             ass_str.append(ast.unparse(ass))
    #
    #                         result_compli_for_else_list.append(
    #                             [repo_name, file_html, cl, me,code[0][0].lineno, "\n".join(ass_str), code[1]])
    #
    #         # print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")
    #     count_repo += repo_exist
    #
    # # a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
    # # print(a)
    # # print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
    # # print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
    # # print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
    # # print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
    # print("count: ", count_repo, code_count, file_count, me_count, all_count_repo, all_file_count, all_me_count)
    # import random

    # random.shuffle(result_compli_for_else_list)
    # util.save_csv(util.data_root + "result_csv/multiple_assign.csv",
    #               result_compli_for_else_list[:400],
    #               ["repo_name", "file_html", "class_name", "me_name","line_no", "old_code", "new_code"])

    # 1 156 2943 100 1 2990 40102 salt
    # 791 2010 103165 1291 800 121348 1192868
    # util.save_csv(util.data_root + "multiple_ass_complicated.csv",
    #               result_compli_for_else_list,
    #               ["repo_name", "file_html", "class_name", "me_name", "for_code", "assign_code"])

# import random
    # random.shuffle(result_compli_for_else_list)
    # util.save_csv(util.data_root+"complicated_code_dir/multip_assign_complicated_sample.csv",result_compli_for_else_list[:385],["repo_name","code1","file_html","file_path"])
   #'''











    # print("----------------------------\n")
    # for code1 in for_else_filter_redundant_code_list:
    #     print("each code1: ", code1[0])



import ast, os, copy
import sys
abs_path=os.path.abspath(os.path.dirname(__file__))
# print("abs_path: ",abs_path)
pack_path="/".join(abs_path.split("/")[:-1])
# print(pack_path)
sys.path.append(pack_path)

import time,util

# from code1 import util
from RefactoringIdioms.extract_simp_cmpl_data import ast_util
from RefactoringIdioms.extract_simp_cmpl_data.extract_compli_truth_value_test_code import decide_compare_complicate_truth_value
from RefactoringIdioms.transform_c_s.transform_truth_value_test_compli_to_simple import transform_c_s_truth_value_test

# from code1.extract_simp_cmpl_data import ast_util
# from code1.extract_simp_cmpl_data.extract_compli_truth_value_test_code import decide_compare_complicate_truth_value
# from pathos.multiprocessing import ProcessingPool as newPool
# from code1.transform_c_s.transform_truth_value_test_compli_to_simple import transform_c_s_truth_value_test

'''
这里在遍历的时候需要记录下 变量T的读写
广度优先遍历
input: 代码片段
对于当前层
if 是赋值语句 (记录 左边的write写的变量T)
    if 后面节点是 for节点
        if for节点的下一个节点是if 语句，其中test中存在T变量
            获得for节点的body
            if exist 涉及T变量的赋值语句
               后面节点是否存在 break节点
    else:
       判断是否T有读操作()

'''


# def traverse_for_else_comp(node,dict_source_code,file_path=None):
#     if
#     if isinstance(node, ast.For):
#
#     pass
class Analyzer(ast.NodeVisitor):
    def __init__(self):
        self.func_def_list = []

    def visit_FunctionDef(self, node):
        self.func_def_list.append(node)

    def visit_If(self, node: ast.If):
        if ast.unparse(node.test) == "__name__ == '__main__'":
            self.func_def_list.append(node)


def get_intersect_vars(assign_list_in_for, vars_if_after_for):
    intersect_flag = 0
    intersect_infor = []

    for ind, node in enumerate(assign_list_in_for):
        if isinstance(node, (ast.Assign,ast.AnnAssign)):
            # print("come assign node: ",node.targets[0].__dict__)
            vars_assign_one = []
            intersect_vars = set([])
            targets=node.targets if isinstance(node,ast.Assign) else [node.target]
            for target in targets:
                vars_assign_one.append(ast.unparse(target))
                # visit_vars(target, vars_assign_one)
            intersect_vars |= set(vars_assign_one) & set(vars_if_after_for)
            if intersect_vars:
                intersect_flag = 1
                intersect_infor.append([node.lineno, node, intersect_vars])

    return intersect_flag, intersect_infor


# def whether_contain_break_and_const_assign(node, if_vars_list):
#     assign_list_in_for = []
#     break_flag = 0
#     for child in ast.iter_child_nodes(node):
#         if hasattr(child, "body"):
#             try:
#                 for e in child.body:
#                     if isinstance(e, ast.Break):
#                         break_flag = 1
#                         break
#                     elif isinstance(e, ast.Assign):
#                         value=e.value
#                         if isinstance(value, ast.Constant):
#                             assign_list_in_for.append(e)
#                     elif isinstance(e, (ast.For, ast.While)):
#                         continue
#                 if break_flag:
#                     break
#             except TypeError:
#                 if isinstance(child.body, ast.Break):
#                     break_flag = 1
#                     break
#                 elif isinstance(child.body, ast.Assign):
#                     assign_list_in_for.append(child.body)
#
#                 # print("child.body: ", child.__dict__, ast.unparse(child),ast.unparse(node))
#         else:
#             continue
#         pass
def whether_is_break(node, if_vars_list):
    # print("the node is: ", node,node.__dict__)
    break_flag = 0
    # if isinstance(node, list):
    #     print("come here")
    #     for e in node:
    #         break_flag=break_flag or whether_is_break(e)
    # else:
    if isinstance(node, (ast.For, ast.While)):
        return 0
    elif isinstance(node, ast.Break):
        return 1
    elif isinstance(node, ast.Assign):
        pass
    elif isinstance(node.ast.If):
        pass
    else:
        for e in ast.iter_child_nodes(node):
            break_flag = break_flag or whether_is_break(e)
    return break_flag


def whether_contain_break_and_const_assign(node, assign_list_in_for, if_list_in_for,break_list_in_for):
    # assign_list_in_for=[]
    # if_list_in_for=[]
    break_flag = 0
    assign_list = []
    if hasattr(node, "body"):
        # print(">>>>>>the node: ",ast.unparse(node))
        for ind, e in enumerate(node.body):

            if isinstance(e, ast.Break):
                    break_list_in_for.append(e)
                    break_flag=1

                    # if 1:#isinstance(node, ast.If):
                    if_list_in_for.append(node)
                    assign_list_in_for.append(assign_list)

                # break
            elif isinstance(e, (ast.Assign,ast.AnnAssign,ast.AugAssign)):
                assign_list.append(e)
            elif isinstance(e, (ast.For, ast.While,ast.AsyncFor)):
                continue
            else:
                # print(">>>>>>the ele: ", ast.unparse(e))
                break_flag = whether_contain_break_and_const_assign(e, assign_list_in_for, if_list_in_for,break_list_in_for) or break_flag

    return break_flag


def visit_vars(target, list_vars):
    # print(">>>>>>>target: ",target.__dict__)
    if isinstance(target, (ast.Name, ast.Subscript, ast.Attribute)):
        list_vars.append(ast.unparse(target))
    else:
        for e in ast.iter_child_nodes(target):
            visit_vars(e, list_vars)


def whether_if_and_contain_var(node):
    flag_if_var = 0
    if_vars_list = None
    if isinstance(node, ast.If):
        flag_if_var = 1
        if_vars_list = node
        # visit_vars(node.test, if_vars_list)
        return flag_if_var, if_vars_list
    else:
        return flag_if_var, if_vars_list


def transform_truth_value_test(test):
    if isinstance(test, ast.Compare):
        if decide_compare_complicate_truth_value(test):
            return transform_c_s_truth_value_test(test)

    return test


def is_same_two_if_test(test_1, test_2):
    def is_same_ops(op1, op2):
        if isinstance(op1, (ast.Eq, ast.Is)) and isinstance(op2, (ast.Eq, ast.Is)):
            return 1
        elif isinstance(op2, (ast.NotEq, ast.IsNot)) and isinstance(op1, (ast.NotEq, ast.IsNot)):
            return 1
        elif isinstance(op1, (ast.Lt)) and isinstance(op2, (ast.Lt)):
            return 1
        elif isinstance(op1, (ast.LtE)) and isinstance(op2, (ast.LtE)):
            return 1
        elif isinstance(op1, (ast.Gt)) and isinstance(op2, (ast.Gt)):
            return 1
        elif isinstance(op1, (ast.GtE)) and isinstance(op2, (ast.GtE)):
            return 1
        return 0

    test_1 = transform_truth_value_test(test_1)
    test_2 = transform_truth_value_test(test_2)
    # print("is_same_two_if_test: ",ast.unparse(test_2),ast.unparse(test_1))
    if isinstance(test_1, ast.Compare) and isinstance(test_2, ast.Compare):
        op1 = test_1.ops[0]
        operators_1 = set([ast.unparse(test_1.left), ast.unparse(test_1.comparators[0])])
        op2 = test_2.ops[0]
        operators_2 = set([ast.unparse(test_2.left), ast.unparse(test_2.comparators[0])])
        # print("compare two ops: ",is_differ_ops(op1, op2))
        if is_same_ops(op1, op2) and operators_1 == operators_2:
            return 1
        pass
    else:
        test_1_str = ast.unparse(test_1)
        test_2_str = ast.unparse(test_2)
        if test_1_str == test_2_str or test_1_str == test_2_str:
            return 1
    return 0


def is_differ_two_if_test(test_1, test_2):
    def is_differ_ops(op1, op2):
        if isinstance(op1, (ast.Eq, ast.Is)) and isinstance(op2, (ast.NotEq, ast.IsNot)):
            return 1
        elif isinstance(op2, (ast.Eq, ast.Is)) and isinstance(op1, (ast.NotEq, ast.IsNot)):
            return 1
        elif isinstance(op1, (ast.Lt)) and isinstance(op2, (ast.GtE)):
            return 1
        elif isinstance(op1, (ast.LtE)) and isinstance(op2, (ast.Gt)):
            return 1
        elif isinstance(op1, (ast.Gt)) and isinstance(op2, (ast.LtE)):
            return 1
        elif isinstance(op1, (ast.GtE)) and isinstance(op2, (ast.Lt)):
            return 1
        return 0

    test_1 = transform_truth_value_test(test_1)
    test_2 = transform_truth_value_test(test_2)

    if isinstance(test_1, ast.Compare) and isinstance(test_2, ast.Compare):
        # print("compare: ",ast.unparse(test_1),ast.unparse(test_2))
        if len(test_1.ops) > 1 and len(test_2.ops) > 1:
            return 0
        op1 = test_1.ops[0]
        operators_1 = set([ast.unparse(test_1.left), ast.unparse(test_1.comparators[0])])
        op2 = test_2.ops[0]
        operators_2 = set([ast.unparse(test_2.left), ast.unparse(test_2.comparators[0])])
        # print("compare two ops: ",is_differ_ops(op1, op2))
        if is_differ_ops(op1, op2) and operators_1 == operators_2:
            return 1
    else:
        test_1_str = ast.unparse(test_1)
        test_2_str = ast.unparse(test_2)
        if "not " + test_1_str == test_2_str or test_1_str == "not " + test_2_str:
            return 1

    return 0


def is_same_ass_init_if_test(assign, test):
    count = 0
    targets = assign.targets if isinstance(assign, ast.Assign) else [assign.target]
    # targets = assign.targets
    value = assign.value
    for tar in targets:
        count += ast_util.get_basic_count(tar)
    if count > 1:
        return 0
    count = ast_util.get_basic_count(value)
    if count > 1:
        return 0
    tree = ast.parse(ast.unparse(targets[0]) + "==" + ast.unparse(value))
    new_test_ass = None
    for node in ast.walk(tree):
        if isinstance(node, ast.Compare):
            return is_same_two_if_test(node, test)

    return 0
    pass


def is_differ_ass_again_if_test(assign, test):
    count = 0
    targets = assign.targets if isinstance(assign, ast.Assign) else [assign.target]
    # targets = assign.targets
    value = assign.value
    for tar in targets:
        count += ast_util.get_basic_count(tar)
    if count > 1:
        return 0
    count = ast_util.get_basic_count(value)
    if count > 1:
        return 0
    tree = ast.parse(ast.unparse(targets[0]) + "==" + ast.unparse(value))

    for node in ast.walk(tree):
        if isinstance(node, ast.Compare):
            # print("is_differ_ass_again_if_test: ",ast.unparse(node),ast.unparse(test))
            return is_differ_two_if_test(node, test)

    return 0
def modify_normalize(tree, child, if_varnode, if_list_in_for, orelse_node,stmts_insert_break):
    child.orelse = orelse_node
    len_body = len(if_list_in_for[0].body)
    for ind_e_if, e_if in enumerate(stmts_insert_break):
        if_list_in_for[0].body.insert(len_body - 1 + ind_e_if, e_if)
    tree.body.remove(if_varnode)
def modify(tree, child, if_varnode, if_list_in_for, flag=0):
    if flag == 0:
        child.orelse = if_varnode.orelse
        len_body = len(if_list_in_for.body)
        for ind_e_if, e_if in enumerate(if_varnode.body):
            if_list_in_for.body.insert(len_body - 1 + ind_e_if, e_if)
        if if_varnode in tree.body:
            tree.body.remove(if_varnode)
    elif flag == 1:
        child.orelse = if_varnode.body
        len_body = len(if_list_in_for.body)

        if if_varnode.orelse:
            for ind_e_if, e_if in enumerate(if_varnode.orelse):
                if_list_in_for.body.insert(len_body - 1 + ind_e_if, e_if)
        if if_varnode in tree.body:
            tree.body.remove(if_varnode)

def remove_assign(tree_copy, child_copy, tree, child, if_vars_list, intersect_infor_ass_list_in_for,
                      intersect_infor_ass_init,ind):
    # print("come remove_assign: ",len(intersect_infor_ass_init))
    if 1:
        # print("len intersect_infor_ass_list_in_for: ",len(intersect_infor_ass_list_in_for))
        count = 0
        for node in ast.walk(child):
            if ast.unparse(node) == if_vars_list[0]:
                count += 1
                if count > len(intersect_infor_ass_list_in_for):
                    break
        else:
            # print("come remove_assign")
            count = 0
            for body in tree.body[ind + 1:]:
                for node in ast.walk(body):
                    if ast.unparse(node) == if_vars_list[0]:
                        count += 1
                        if count > 1:
                            break
                if count > 1:
                    break
            else:
                for e in ast.walk(child_copy):
                    if hasattr(e, "body") and isinstance(e.body,list):
                        # print("body: ",e,ast.unparse(e))
                        for e_each in e.body:
                            for ass_each_list in intersect_infor_ass_list_in_for:
                                if e_each == ass_each_list[0][1]:
                                    e.body.remove(e_each)
                for e in ast.walk(tree_copy):
                    if hasattr(e, "body")  and isinstance(e.body,list):
                        # print("body: ",e,ast.unparse(e))
                        for node in e.body:
                            if ast.unparse(intersect_infor_ass_init[0][1]) == ast.unparse(node):
                                e.body.remove(node)
                                break
                        else:
                            continue
                        break
    # print("new_tree: ", ast.unparse(tree))
def traverse_cur_layer(tree, code_list, ass_init_list):
    # print(">>>>>>>>>assign_list: ", ass_init_list)
    if hasattr(tree, "body"):
        for ind,child in enumerate(tree.body):
            tree_copy = copy.deepcopy(tree)
            child_copy = tree_copy.body[ind]
            if isinstance(child, (ast.For, ast.While)):
                if child.orelse:
                    traverse_cur_layer(child, code_list, ass_init_list)
                    continue
                if ind + 1 >= len(tree.body):
                    traverse_cur_layer(child, code_list, ass_init_list)
                    continue
                if_vars_list, intersect_infor, assign_list_in_for = [], [], []
                # print("tree_copy: ",tree_copy,tree.body,ast.unparse(child),child)
                next_child = tree_copy.body[ind + 1]

                flag_if_var, if_varnode = whether_if_and_contain_var(next_child)

                if not if_varnode:
                    traverse_cur_layer(child, code_list, ass_init_list)
                    continue
                # print("it is if node after for stmt: ", flag_if_var, ast.unparse(next_child), if_vars_list)
                all_assign_list_in_for = []
                if_list_in_for = []
                break_list_in_for=[]
                break_flag = whether_contain_break_and_const_assign(child_copy, all_assign_list_in_for, if_list_in_for,break_list_in_for)


                if break_flag:

                    flag_simplify = 0
                    if if_varnode.orelse:
                        for if_list_each in if_list_in_for:
                            if hasattr(if_list_each, "test") and if_list_each.test and is_same_two_if_test(
                                if_varnode.test, if_list_each.test):
                                # modify(tree_copy, child_copy, if_varnode, if_list_each, 0)
                                flag_simplify=1
                            else:
                                flag_simplify=0
                                break
                        else:
                            for if_list_each in if_list_in_for:
                                modify(tree_copy, child_copy, if_varnode, if_list_each, 0)


                    if not flag_simplify:
                        for if_list_each in if_list_in_for:
                            if not hasattr(if_list_each, "test") or (hasattr(if_list_each, "test") and if_list_each.test and not is_differ_two_if_test(
                                if_varnode.test, if_list_each.test)):
                                break
                        else:
                            for if_list_each in if_list_in_for:
                                modify(tree_copy, child_copy, if_varnode, if_list_each, 1)
                            flag_simplify=1


                    assign_flag=0

                    # print("ass_init_list: ",ass_init_list)
                    if not flag_simplify:
                            # if_test and assign_again
                            flag_simplify=1
                            for ind_ass,assign_list_in_for in enumerate(all_assign_list_in_for):
                                for ass in assign_list_in_for:
                                    if ass_init_list and ast.unparse(ass_init_list[-1].value) != ast.unparse(ass.value):
                                        if is_differ_ass_again_if_test(ass, if_varnode.test):
                                            break
                                        elif if_varnode.orelse and is_same_ass_init_if_test(ass, if_varnode.test):
                                            break
                                        else:
                                            continue
                                else:
                                    flag_simplify = 0
                                    break
                            if flag_simplify:
                                for ind_ass, assign_list_in_for in enumerate(all_assign_list_in_for):
                                    for ass in assign_list_in_for:
                                        if ass_init_list and ast.unparse(ass_init_list[-1].value) != ast.unparse(ass.value):
                                            if is_differ_ass_again_if_test(ass, if_varnode.test):
                                                modify(tree_copy, child_copy, if_varnode, if_list_in_for[ind_ass], 1)
                                                break
                                            elif if_varnode.orelse and is_same_ass_init_if_test(ass, if_varnode.test):
                                                modify(tree_copy, child_copy, if_varnode, if_list_in_for[ind_ass], 0)
                                                break
                                assign_flag = 1
                                # print("new_tree: ",ast.unparse(tree_copy))

                            # for ind_ass,assign_list_in_for in enumerate(all_assign_list_in_for):
                            #     for ass in assign_list_in_for:
                            #         if ast.unparse(ass_init_list[-1].value)!=ast.unparse(ass.value):
                            #             if is_differ_ass_again_if_test(ass, if_varnode.test):
                            #                 flag_simplify = 1
                            #                 for if_list_each in if_list_in_for:
                            #                     modify(tree_copy, child_copy, if_varnode, if_list_each, 1)
                            #                 # print("assig list in for new_tree 1: ", ast.unparse(tree_copy))
                            #                 break
                            #             elif if_varnode.orelse and is_same_ass_init_if_test(ass, if_varnode.test):
                            #                 for if_list_each in if_list_in_for:
                            #                     modify(tree_copy, child_copy, if_varnode, if_list_each, 0)
                            #                 # print("assig list in for  new_tree 0: ", ast.unparse(tree_copy))
                            #                 flag_simplify = 1
                            #                 break
                            #             else:
                            #                 flag_simplify = 0
                            #                 break

                        # print("new tree: ",ast.unparse(tree_copy),if_list_in_for)
                    visit_vars(if_varnode.test, if_vars_list)
                    # for ass_ini in ass_init_list:
                    #     print("ass_ini: ",ast.unparse(ass_ini))
                    intersect_flag_ass_init, intersect_infor_ass_init = get_intersect_vars(ass_init_list,if_vars_list)
                    # for ass_ini in intersect_infor_ass_init:
                    #     print("ass_ini: ",ast.unparse(ass_ini[1]))
                    intersect_flag_ass_list_in_for, intersect_infor_ass_list_in_for =0,[]
                    for assign_list_in_for in all_assign_list_in_for:
                        flag_inn,info_inn=get_intersect_vars(assign_list_in_for, if_vars_list)

                        if not flag_inn:
                            break
                        intersect_infor_ass_list_in_for.append(info_inn)
                    else:
                        intersect_flag_ass_list_in_for=1

                    # print("if_vars_list: ",if_vars_list)
                    # for ass in ass_init_list:
                    #     print("ass: ",ast.unparse(ass))
                    # print("intersect_flag_ass_init: ",ass_init_list,intersect_flag_ass_init,intersect_infor_ass_init)
                    # if intersect_infor_ass_init and intersect_flag_ass_list_in_for:
                    #     assign_init = ast.unparse(intersect_infor_ass_init[-1][1])
                    #     print("intersect_flag_ass_init: ",intersect_infor_ass_init,assign_init)

                    if not flag_simplify and intersect_infor_ass_init and intersect_flag_ass_list_in_for:
                        if is_same_ass_init_if_test(intersect_infor_ass_init[-1][1], if_varnode.test):
                            # print("yes  if_test and assign init could be simplified to for else: ",ast.unparse(tree_copy))
                            # print("child copy: ",ast.unparse(child_copy),"-------\n",ast.unparse(if_varnode),if_list_in_for)
                            for if_list_each in if_list_in_for:
                                modify(tree_copy, child_copy, if_varnode, if_list_each, 1)
                            # print("assign init new_tree: ",ast.unparse(tree_copy))
                            flag_simplify = 1
                            assign_flag = 1
                        elif  if_varnode.orelse and  is_differ_ass_again_if_test(intersect_infor_ass_init[-1][1], if_varnode.test):
                            for if_list_each in if_list_in_for:
                                modify(tree_copy, child_copy, if_varnode, if_list_each, 0)
                            # print("assign init new_tree: ", ast.unparse(tree_copy))
                            flag_simplify = 1
                            assign_flag = 1
                    if flag_simplify and assign_flag:
                            remove_assign(tree_copy,child_copy,tree,child,if_vars_list,intersect_infor_ass_list_in_for,intersect_infor_ass_init,ind)



                    if flag_simplify:
                        # print("old tree: ",ast.unparse(tree))
                        # print("new tree: ", ast.unparse(tree_copy), if_list_in_for)
                        code_list.append([tree,tree_copy,break_list_in_for])
                        # code_list.append([ ast.unparse(tree),ast.unparse(tree_copy),break_list_in_for[0].lineno])
                        # code_list.append(
                        #     [assign_init + "\n" + ast.unparse(child) + "\n" + ast.unparse(next_child), child.lineno,
                        #      next_child.lineno])

                # if break_flag and intersect_flag_ass or intersect_flag_if and break_flag:
                #     code_list.append([intersect_infor_ass_init[-1],child,next_child,])
                #     code_list.append([ast.unparse(child) + "\n" + ast.unparse(next_child),child.lineno,next_child.lineno])
            else:
                for e_chi in ast.walk(child_copy):
                    if isinstance(e_chi, (ast.Assign,ast.AnnAssign)) and e_chi not in ass_init_list  :
                        # print("else: ",ast.unparse(e_chi))
                        ass_init_list.append(e_chi)
                # print("assign_list: ",ass_init_list)

            traverse_cur_layer(child, code_list, ass_init_list)


def save_repo_for_else_complicated(repo_name):
    start=time.time()
    count_complicated_code = 0
    print("come the repo: ", repo_name)
    one_repo_for_else_code_list = []
    # if os.path.exists(save_complicated_code_dir + repo_name + ".json"):
    #     print("the repo has been saved before")
    #     return None

    for ind,file_info in enumerate(dict_repo_file_python[repo_name]):



        file_path = file_info["file_path"]
        # if file_path!="/mnt/zejun/smp/data/python_repo_1000/VideoPose3D//run.py":
        #     continue
        file_html = file_info["file_html"]


        try:
            content = util.load_file(file_path)
        except:
            print(f"{file_path} is not existed!")
            continue

        # print("content: ",content)
        try:
            file_tree = ast.parse(content)
            ana_py = Analyzer()
            ana_py.visit(file_tree)
            code_list = []
            # print("func number: ",file_html,len(ana_py.func_def_list))
            for tree in ana_py.func_def_list:
                # print("tree_ func_name",tree.__dict__)
                new_code_list = []
                ass_init_list = []
                traverse_cur_layer(tree, new_code_list, ass_init_list)

                if new_code_list:
                    # print(">>>>>>>>>>file_html: ", file_html)
                    # print("code1 list:\n", new_code_list)
                    code_list.extend(new_code_list)
            if code_list:
                one_repo_for_else_code_list.append([code_list, file_path, file_html])
                # if len(one_repo_for_else_code_list)>2:
                #     break

        except SyntaxError:
            print("the file has syntax error")
            continue
        except ValueError:
            print("the file has value error: ", content, file_html)
            continue
        # break
    end = time.time()
    #'''
    if one_repo_for_else_code_list:
        count_complicated_code = count_complicated_code + len(one_repo_for_else_code_list)
        # print("it exists for else complicated code1: ", len(one_repo_for_else_code_list))
        util.save_pkl(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
        # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
        print(end-start," save successfully! ", save_complicated_code_dir + repo_name)
    else:
        print(end-start," the repo has no for else")
        util.save_pkl(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
        # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
    #'''
    return count_complicated_code


if __name__ == '__main__':
    code = '''
# if match:
#     skip = True
#     for pat in self.patterns:
#         if not pat.exclusion:
#             continue
#         if pat.cleaned_pattern.startswith(normalize_slashes(fpath)):
#             skip = False
#             break
#     if skip:
#         continue
        
# for current_package in self._current_packages:
#     found = False
#     for (result_package, _) in self._result_packages:
#         if current_package.name == result_package.name:
#             found = True
#             break
#     if not found:
#         for installed_package in self._installed_packages:
#             if installed_package.name == current_package.name:
#                 operations.append(Uninstall(current_package))
# def __get_path_from_ref(self, ref):
#     refs = ref.split('/')
#     for (i, ref) in enumerate(refs):
#         if ref.endswith('.kp'):
#             break
#     if not ref.endswith('.kp'):
#         return None
#     return '/'.join(refs[:i + 1])
    
# crumb = None
# sess = requests.Session()
# for i in range(self.retries + 1):
#         resp = sess.get(url, **sesskwargs)
#         if resp.status_code != requests.codes.ok:
#             continue
#         txt = resp.text
#         i = txt.find('CrumbStore')
#         istart = txt.find('"', i + len('crumb') + 1)
#         if istart == -1:
#             continue
#         istart += 1
#         iend = txt.find('"', istart)
#         if iend == -1:
#             continue
#         crumb = txt[istart:iend]
#         crumb = crumb.encode('ascii').decode('unicode-escape')
#         break
# if crumb is None:
#         self.error = 'Crumb not found'
#         self.f = None
#         return
    
    
# is_choice = False
# if len(choice_list) == 0:
#     is_choice = True
# for ch in choice_list:
#     if ch in title:
#         is_choice = True
#         break
# if is_choice == False:
#     return False
# is_ban = False
# for ba in ban_list:
#     if ba in title:
#         is_ban = True
#         break
# if is_ban == True:
#     return False
# return True

# def trigger(self, char, *args, **kwargs):
#     if isinstance(char, six.string_types) and len(char) == 1:
#         char = ord(char)
#     func = None
#     for controller in self.parents:
#         func = controller.character_map.get((self.last_char, char))
#         if func:
#             break
#         func = controller.character_map.get(char)
#         if func:
#             break
#     if func:
#         self.last_char = None
#         return func(self.instance, *args, **kwargs)
#     else:
#         self.last_char = char
#         return None

# root_hierarchical_hash = None
# if hierarchical_hashes:
#     hierarchical_grouphashes = {h.hash: h for h in GroupHash.objects.filter(project=project, hash__in=hierarchical_hashes)}
#     for hash in reversed(hierarchical_hashes):
#         group_hash = hierarchical_grouphashes.get(hash)
#         if group_hash is not None and group_hash.state == GroupHash.State.SPLIT:
#             found_split = True
#             break
#         root_hierarchical_hash = hash
#         if group_hash is not None:
#             all_grouphashes.append(group_hash)
#     if root_hierarchical_hash is None:
#         root_hierarchical_hash = hierarchical_hashes[-1]
#         group_hash = hierarchical_grouphashes.get(root_hierarchical_hash)
#         if group_hash is not None:
#             all_grouphashes.append(group_hash)
# for i in range(2):
#     print(i)
#     break
# if a:
#     print("1")

# response = None
# callback, callback_args, callback_kwargs = self.resolve_request(request)
# 
# # Apply view middleware.
# for middleware_method in self._view_middleware:
#     response = await middleware_method(request, callback, callback_args, callback_kwargs)
#     if response:
#         break
# 
# if response is None:
#     # print("test_if")
#     wrapped_callback = self.make_view_atomic(callback)    
# else:
#     print("test")
#     print("test2")



# _checkpoint_callback = None
# require_checkpoint = False
# 
# with remote_store.get_local_output_dir() as run_output_dir:
#     logs_path = os.path.join(run_output_dir, remote_store.logs_subdir)
#     os.makedirs(logs_path, exist_ok=True)
#     print(f"Made directory {logs_path} for horovod rank {hvd.rank()}")
#     ckpt_dir = run_output_dir
#     ckpt_filename = remote_store.checkpoint_filename
# 
#     if logger is None:
#         # Use default logger if no logger is supplied
#         train_logger = TensorBoardLogger(logs_path)
#         print(f"Setup logger: Using TensorBoardLogger: {train_logger}")
# 
#     elif isinstance(logger, CometLogger) and logger._experiment_key is None:
#         # Resume logger experiment key if passed correctly from CPU.
#         train_logger = CometLogger(
#             save_dir=logs_path,
#             api_key=logger.api_key,
#             experiment_key=logger_experiment_key,
#         )
# 
#         print(f"Setup logger: Resume comet logger: {vars(train_logger)}")
#     else:
#         # use logger passed in.
#         train_logger = logger
#         train_logger.save_dir = logs_path
#         print(f"Setup logger: Using logger passed from estimator: {train_logger}")
# 
#     # Lightning requires to add checkpoint callbacks for all ranks.
#     # Otherwise we are seeing hanging in training.
#     for cb in callbacks:
#         if isinstance(cb, ModelCheckpoint):
#             cb.dirpath = ckpt_dir
#             cb.filename = ckpt_filename
#             _checkpoint_callback = cb
#             require_checkpoint = True
#             break
#     if not _checkpoint_callback:
#         # By default 'monitor'=None which saves a checkpoint only for the last epoch.
#         _checkpoint_callback = ModelCheckpoint(dirpath=ckpt_dir,
#                                                filename=ckpt_filename,
#                                                verbose=True) 
#     else:
#         print("the else 1")
#         print("the else 2")
       
# 
# for module_name in relevant_modules:
#     relevant_keys = set([key for key in state_dict.keys() if key.startswith(module_name + '.')])
#     if relevant_keys:
#         state_dict = {key.replace(module_name + '.', '', 1): value for (key, value) in state_dict.items() if key in relevant_keys}
#         found = True
#         break
# if not found:
#     warnings.warn(f'{relevant_modules} was not found at top level of state_dict!', UserWarning)
# else:
#     print("the else 1")
#     print("the else 2")
# for new_loc in new_locs:
#     new_loc_nes = [xx for xx in [(new_loc[0] + 1, new_loc[1]), (new_loc[0] - 1, new_loc[1]), (new_loc[0], new_loc[1] + 1), (new_loc[0], new_loc[1] - 1)] if xx[0] >= 0 and xx[0] < my_fpath_map.shape[0] and (xx[1] >= 0) and (xx[1] < my_fpath_map.shape[1])]
#     if fpath_map is not None and np.sum([fpath_map[nlne[0], nlne[1]] for nlne in new_loc_nes]) != 0:
#         break_flag = True
#         break
#     if my_npath_map[new_loc[0], new_loc[1]] != -1:
#         continue
#     if npath_map is not None and npath_map[new_loc[0], new_loc[1]] != edge_id:
#         break_flag = True
#         break
#     fpath.append(new_loc)
# if break_flag is True:
#     break    


# for controller in self.parents:
#     func = controller.character_map.get((self.last_char, char))
#     if func:
#         break
#     func = controller.character_map.get(char)
#     if func:
#         break
# if func:
#     self.last_char = None
#     return func(self.instance, *args, **kwargs)
# else:
#     self.last_char = char
#     return None
found = False
for a in action_filter:
    if action.startswith(a):
        found = True
        break
if not found:
    continue

# for (host, slots) in host_list:
#     slots = int(slots)
#     if slots > lsf.LSFUtils.get_num_gpus():
#         raise ValueError("Invalid host input, slot count for host '{host}:{slots}' is greater than number of GPUs per host '{gpus}'.".format(host=host, slots=slots, gpus=lsf.LSFUtils.get_num_gpus()))
#     needed_slots = min(slots, remaining_slots)
#     validated_list.append((host, needed_slots))
#     remaining_slots -= needed_slots
#     if remaining_slots == 1:
#         break
# if remaining_slots != 1:
#     raise ValueError('Not enough slots on the hosts to fulfill the {slots} requested.'.format(slots=settings.num_proc))

# for f in possible_font_names:
#     pdf_text_font = pdf_text_fonts.get(f, None)
#     if pdf_text_font is not None:
#         font_key = f
#         break
# if pdf_text_font:
#     font = self.pdf_base.copy_foreign(pdf_text_font)
# a=1
# b=2
# c,d=1,2
# for i in range(2):
#     a=1
#     if i>1:
#         if i>2:
#             a=3
#         a=2
#         break 
# if a==2:
#     print(1)
# print("1")   
# def a():
#     return 1
# a=2
# for i in range(5):
#     a=1
#     break
# if a:
#     print("test")
# for i in range(2):
#     if i>2:
#         a=1
#         for j in range(3):
#             if j>2:
#                 break
# if a:
#     print("it should not a 可简化的for else代码")
# if a>2:
#     for i in range(2):
#         break
# elif a>3:
#     print("test")
# def func2():
#     while start < len(chars):
#         end = len(chars)
#         cur_substr = None
#         while start < end:
#             substr = ''.join(chars[start:end])
#             if start == 0:
#                 substr = sentencepiece_prefix + substr
#             if start > 0:
#                 substr = prefix + substr
#             if substr in vocab:
#                 cur_substr = substr
#                 break
#             end -= 1
#         if cur_substr is None:
#             is_bad = True
#             break
#         sub_tokens.append(cur_substr)
#         sub_pos.append((start, end))
#         start = end
#     if is_bad:
#         return ([unk_token], [(0, len(chars))])
#     else:
#         return (sub_tokens, sub_pos)
# def func():
#     for nexttag in tag_iter:
#         tagname = nexttag.tag
#         attributes = nexttag.attributes
#         if tagname == 'a' and (nexttag.tag_type == CLOSE_TAG or (attributes.get('href') and (not attributes.get('href', '').startswith('#')))):
#             if astart:
#                 yield mklink(ahref, body[astart:nexttag.start], nofollow)
#                 astart = ahref = None
#                 nofollow = False
#             href = attributes.get('href')
#             if href:
#                 ahref = href
#                 astart = nexttag.end
#                 nofollow = attributes.get('rel') == u'nofollow'
#         elif tagname == 'head':
#             for nexttag in tag_iter:
#                 tagname = nexttag.tag
#                 if tagname == 'head' and nexttag.tag_type == HtmlTagType.CLOSE_TAG or tagname == 'body':
#                     break
#                 if tagname == 'base':
#                     href = nexttag.attributes.get('href')
#                     if href:
#                         joined_base = urljoin(htmlpage.url, href.strip().strip(), htmlpage.encoding)
#                         base_href = remove_entities(joined_base, encoding=htmlpage.encoding)
#                 elif tagname == 'meta':
#                     attrs = nexttag.attributes
#                     if attrs.get('http-equiv') == 'refresh':
#                         m = _META_REFRESH_CONTENT_RE.search(attrs.get('content') or '')
#                         if m:
#                             target = m.group('url')
#                             if target:
#                                 yield mklink(target)
#                 elif tagname == 'link':
#                     href = nexttag.attributes.get('href')
#                     if href:
#                         yield mklink(href)
#         elif tagname == 'area':
#             href = attributes.get('href')
#             if href:
#                 nofollow = attributes.get('rel') == 'nofollow'
#                 yield mklink(href, attributes.get('alt', ''), nofollow)
#         elif tagname in ('frame', 'iframe'):
#             target = attributes.get('src')
#             if target:
#                 yield mklink(target)
#         elif 'onclick' in attributes:
#             match = _ONCLICK_LINK_RE.search(attributes['onclick'] or '')
#             if not match:
#                 continue
#             target = match.group('url')
#             nofollow = attributes.get('rel') == u'nofollow'
#             yield mklink(target, nofollow=nofollow)
#     if astart:
#         yield mklink(ahref, htmlpage.body[astart:])
'''
    #     code1 = '''
    # for i in range(2):
    #     a=1
    #     b=2
    #     for j in range(3):
    #         w=8
    #         break
    #     if i>1 and j>3 and i!=0:
    #         if i>2 and call(1,t,*T):
    #             b=3
    #             break
    #     c=3
    #     '''

    tree = ast.parse(code)
    # for node in ast.walk(tree):
    #     if isinstance(node,(ast.For,ast.While)):
    #         print("-------------------------")
    #         assign_list_in_for,if_list_in_for=[],[]
    #         whether_contain_break_and_const_assign(node, assign_list_in_for, if_list_in_for)
    #         print(assign_list_in_for,if_list_in_for)
    #         # assign_list_in_for=[]
    #         # if_list_in_for=[]
    #         # break_flag=whether_contain_break_and_const_assign(node, assign_list_in_for,if_list_in_for)
    #         #
    #         # print(break_flag,assign_list_in_for,if_list_in_for)
    #     elif isinstance(node,ast.If):
    #         if_vars=[]
    #         visit_vars(node.test,if_vars)
    #         print("if vars: ",if_vars)
    code_list = []
    ass_init_list = []
    traverse_cur_layer(tree, code_list, ass_init_list)
    print("*****************: ", code_list)
    # code_list=[]
    # tree=ast.parse(code1)
    # traverse_cur_layer(tree,code_list)
    # print(code_list)

    # for_else_filter_redundant_code_list = get_for_else_complicate_code(file_tree)
    # print(for_else_filter_redundant_code_list)
    # for one_code in for_else_filter_redundant_code_list:
    #     print("-----------------------",one_code)
    #     print(one_code[0])

    # file_tree = ast.parse(code1)
    # ana_py = Analyzer()
    # ana_py.visit(file_tree)
    # print("fun number: ",len(ana_py.func_def_list))

    # save_complicated_code_dir= util.data_root + "complicated_code_dir_1000_star/for_else_assign_is_const_complicated/"
    save_complicated_code_dir = util.data_root + "transform_complicate_to_simple/for_else_improve_5/"
    dict_repo_file_python= util.load_json(util.data_root, "python3_1000repos_files_info")

    # dict_repo_file_python = util.load_json(util.data_root, "jupyter3_repos_files_info")

    repo_list = []
    count_file = 0
    for ind, repo_name in enumerate(dict_repo_file_python):
        # if repo_name=="salt":
        #     continue
        # if os.path.exists(save_complicated_code_dir + repo_name + ".json"):
        #     continue
        # if ind>30:
        #     break
        # print("repo_name: ", repo_name)
        repo_list.append(repo_name)
        # break
    print("len: ", len(repo_list))
    # save_repo_for_else_complicated(repo_list[0])
    # savetransform_c_s_repo_for_else_complicated(repo_list[0])
    '''
        for file_info in dict_repo_file_python[repo_name]:

            file_path = file_info["file_path"]
            # if file_path!="/mnt/zejun/smp/data/python_repo_1000/VideoPose3D//run.py":
            #     continue
            file_html = file_info["file_html"]
            #print("come this file: ", file_path)
            try:
                content = util.load_file(file_path)
            except:
                print(f"{file_path} is not existed!")
                continue

            #print("content: ",content)
            try:
                file_tree = ast.parse(content)
                count_file+=1
            except:
                continue
    print("number of repos: ", len(repo_list),count_file)
    '''
    '''
    count_complicated_code=0
    for ind,repo_name in enumerate(dict_repo_file_python):
        print("come repo: ",repo_name)
        count_complicated_code+=save_repo_for_else_complicated(repo_name)
        # break
    print("count_complicated_code: ",ind,count_complicated_code)
    '''
    # for repo in repo_list:
    #     # if repo!="django-extensions":#"keras-bert":#"sympy":#"Real-Time-Voice-Cloning":#"keras-bert":#"Legofy":
    #     #     continue
    #     save_repo_for_else_complicated(repo)

    '''
    pool = newPool(nodes=30)
    pool.map(save_repo_for_else_complicated, repo_list)  # [:3]sample_repo_url ,token_num_list[:1]
    pool.close()
    pool.join()
    # print("len all_files: ", len(all_files))
    '''

    # repo_files_info = util.load_json(util.data_root, "dict_1000_repo_files_info")
    # repo_star_info = util.load_json(util.data_root, "dict_1000_repo_star_info")
    # repo_contributor_info = util.load_json(util.data_root, "dict_1000_repo_contributor_info")
    files_num_list = []
    star_num_list = []
    contributor_num_list = []
    count = 0
    count_repo=0
    file_num_list = []
    file_list = set([])
    repo_code_num=dict()
    result_compli_for_else_list = []
    for file_name in os.listdir(save_complicated_code_dir):
        repo_name = file_name[:-5]
        # files_num_list.append(repo_files_info[repo_name])
        # star_num_list.append(repo_star_info[repo_name])
        # contributor_num_list.append(repo_contributor_info[repo_name])

        complicate_code = util.load_json(save_complicated_code_dir, file_name[:-5])
        if complicate_code:
            count_repo += 1
        count_code_fra=0
        for code_list, file_path, file_html in complicate_code:

            count += len(code_list)
            count_code_fra+=len(code_list)
            file_num_list.append(len(code_list))
            for code in code_list:
                repo_name = file_html.split("/")[4]
                file_list.add(file_html)
                result_compli_for_else_list.append([repo_name, code[0],code[1], file_html, file_path])
            print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")

            #     break
            # break
        # if complicate_code:
        #     repo_code_num[repo_name]=count_code_fra
        #     print(f"{repo_name} has  {count_code_fra} code1 fragments")
        # break
    # a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
    # print(a)
    # print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
    # print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
    # print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
    # print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
    print("count: ", count_repo,count, len(os.listdir(save_complicated_code_dir)), len(file_list))
    # util.save_csv(util.data_root + "transform_complicate_to_simple/for_else_improve_4.csv", result_compli_for_else_list,
    #               ["repo_name", "old_code","new_code", "file_html", "file_path"])
    # util.save_csv(util.data_root+"complicated_code_dir_1000_star/for_else_iftest_intersect_iftest_assign.csv",result_compli_for_else_list,["repo_name","code1","file_html","file_path"])

    # util.save_csv(util.data_root + "complicated_jupyter_code_dir/for_else.csv", result_compli_for_else_list,
    #               ["repo_name", "code1", "file_html", "file_path"])

    # print("----------------------------\n")
    # for code1 in for_else_filter_redundant_code_list:
    #     print("each code1: ", code1[0])
